/*  Copyright (C) 2010-2024 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef CALCULATEPOLAR_SRC_METHODS_SEMIEMPIRICALHIGHLIFTADAPTION_H_
#define CALCULATEPOLAR_SRC_METHODS_SEMIEMPIRICALHIGHLIFTADAPTION_H_

#include "../polar.h"

#include <moduleBasics/runtimeIO.h>
#include <aircraftGeometry2/airfoil_surface.h>

#include <memory>
#include <string>
#include <vector>
#include <unordered_map>

class semiEmpiricalHighLiftAdaption{
 public:
    bool hasStabilizer;
    bool do_individual_hl_calibration;
    bool apply_manual_CLmax_hl_values;
    geom2::MultisectionSurface<geom2::AirfoilSection> theStabilizer;
    double CLmaxTO_req;  //TBA
    double CLmaxLDG_req;  //TBA
    double stepWidthCL;
    double CDmin_inc;
    double kFactor_inc;
    double CLatCDmin_inc;
    double reductionDragCountsHighLift;
    double factorDragHighLift;
    double dCMtodCL_inc;
    double CMatCL0_inc;
    double dCMtodAoA;
    double dCLtodAoA;
    double CLatAoA0;
    double CMatAoA0;
    class WingDevice {
     public:
        std::string name;                   /**< Name/Identifier of control surface [-]*/
        double maxPositiveDeflection;
        double maxNegativeDeflection;
        double innerSpan_rel;               /**< Relative inner span control surface [-] */
        double outerSpan_rel;               /**< Relative outer span control surface [-] */
        double innerRelFwdChordPosition;
        double innerRelAftChordPosition;
        double outerRelFwdChordPosition;
        double outerRelAftChordPosition;

        double innerSpan_abs;   /**< Absolute inner span control surface [m] */
        double outerSpan_abs;   /**< Absolute outer span control surface [m] */
        double span;            /**< With (span) of element [m] */
        double c_rel_avg;       /**< Average chord depth [m] */
        double x_rel_hinge;     /**< rel. wing chord position of hingeline (Raymer) [m] */
        double phiHL;           /**< Sweep of hingeline [deg] */
        double coveredA;        /**< Raymer, flappedArea [m^2] */
        double deltaMaxCL;      /**< Maximum lift coefficient difference [-] */

        /** \brief Constructor of HighLiftDevice class
         *  \param &aWing Reference to liftingSurface object the high lift device on which it is installed
         *  \param &aHLDevice Reference to controlSurface object
         *  \param TE_element Switch whether the high lift device is a trailing edge element
         */
        // HighLiftDevice(const liftingSurface &aWing, const liftingSurface::controlSurface &aHLDevice, bool TE_element);
        /** \param Virtual destructor of HighLiftDevice class
         */
        virtual ~WingDevice() {}
    };
    std::vector <WingDevice> wingDevices;
    struct highLiftAdaption {
    double factorLE;  /**< Leading edge coefficient [-] */
    double factorTE;  /**< Leading edge coefficient [-] */
    double deltaCDmin;  /**< Minimum delta drag coefficient [-] */
    double deltaCDgear;  /**< Delta drag coefficient (gear) [-] */
    double deltaCLatCDmin;  /**< Delta lift coefficient at minimum drag [-] */
    };
    std::vector <WingDevice> leadingEdgeHighLiftDevices;
    std::vector <WingDevice> trailingEdgeHighLiftDevices;
    std::unordered_map<std::string, highLiftAdaption> theHighliftAdaption;
    struct hl_polar_calibration_values {
        double CL_max;          /**< maximum CL value which should be reached */
        double delta_CD;        /**< Constant delta for drag calibration */
        double factor_CD;       /**< Factor for drag calibration */
    };
    hl_polar_calibration_values aHLPolarCalibrationContainer;
    std::unordered_map<std::string, hl_polar_calibration_values> hl_polar_calibration_data; /**< Container for storing all high lift polar calibration and correction values */    
    void runHighLiftAdaption();
    void setHighLiftDevices();
    void processHighLiftDevices();
    void setDeltaCLMaxDevices();
    auto setRaymerHighLiftAdaptionValues() -> std::unordered_map<std::string, highLiftAdaption>;
    Polar calculateHighLiftPolar(std::string config, const double CoG_X_position, const double currentIStabAngle);
    double getDeltaCMHighLift(const double CoG_X_Position, double Clift, double Mach, double CLmaxClean, double CLmaxHighLift);
    double getAlphaAtCL(Polar aPolar, double aCL);
    double getCLMax(Polar aPolar);
    Polar theCleanPolar;
    explicit semiEmpiricalHighLiftAdaption(const std::shared_ptr<RuntimeIO>& rtIO, const geom2::MultisectionSurface<geom2::AirfoilSection> mainWing);
    virtual ~semiEmpiricalHighLiftAdaption() {}

 private:
geom2::MultisectionSurface<geom2::AirfoilSection> theMainWing;
};

#endif  // CALCULATEPOLAR_SRC_METHODS_SEMIEMPIRICALHIGHLIFTADAPTION_H_
