def calculate_maintenance_costs(dict_ac_data, dict_operating_costs, runtime_output):
    """Calculate annual maintenance costs.

    // TODO check cost burden (why not 2 as in PS?)
    Maintenance cost are split into
        - flight cycle dependent costs, reflecting primarily structural fatigue and overhaul burden,
        - flight hour dependent costs, which consider primarily wear and the associated line maintenance work, and
        - calendar time dependent costs, which are a constant share reflecting for example the rectification of
        corrosion during overhaul, the latter being route independent and thus considered as constant.
    Following the JADC-method an approximation for maintenance cost per flight cycle is given by the sum of three parts:
        - 'airframe_material': Airframe material maintenance costs (repair and replacement)
        - 'airframe_personnel': Airframe personnel maintenance costs (inspection and repair)
        - 'engine_total': Engine total maintenance costs

    This function adds the 'maintenance_costs' dict to the 'dict_operating_costs' that contains the following
    parameters:
        - float maintenance_costs: Maintenance costs per year in EUR

    :param dict dict_ac_data: Dict containing parameters and values from aircraft exchange and module configuration file
    :param dict dict_operating_costs: Dict containing parameters from cost estimation calculation
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return dict dict_operating_costs: Dict containing parameters from cost estimation calculation
    """

    # Extract necessary variables from 'dict_ac_data'.
    airframe_repair_costs_per_flight = dict_ac_data['airframe_repair_costs_per_flight']
    cost_burden = dict_ac_data['cost_burden']
    factor_engine_maintenance = dict_ac_data['factor_engine_maintenance']
    flight_time = dict_ac_data['flight_time']/3600
    m_operating_empty = dict_ac_data['m_operating_empty']
    rate_labor = dict_ac_data['rate_labor']
    ratio_operating_empty_mass = dict_ac_data['ratio_operating_empty_mass']
    static_thrust_per_engine = dict_ac_data['static_thrust_per_engine']
    # Extract necessary values from 'dict_operating_costs'.
    n_annual_flights = dict_operating_costs['dependent_parameter']['flights_per_year']
    newton_to_tonnes = 0.000101971621

    # Definition of function variables.
    ACCELERATION_OF_GRAVITY = 9.80665

    # Calculate maintenance costs.
    airframe_material = ((m_operating_empty / 1000) * (0.2 * flight_time + 13.7)
                         + (airframe_repair_costs_per_flight*ratio_operating_empty_mass))
    airframe_personnel = rate_labor * (1 + cost_burden) * ((0.655 + 0.01 * (m_operating_empty / 1000))
                                                           * flight_time + 0.254 + 0.01 * (m_operating_empty / 1000))
    engine_maintenance_cost = []
    for engine_ID in static_thrust_per_engine.keys():
        engine_entity_maintenance_cost = (1.5 * (static_thrust_per_engine[engine_ID]*newton_to_tonnes)
                                          / ACCELERATION_OF_GRAVITY + 30.5 * flight_time + 10.6)
        engine_maintenance_cost.append(engine_entity_maintenance_cost)
    engine_total = sum(engine_maintenance_cost) * factor_engine_maintenance
    maintenance_costs = (airframe_material + airframe_personnel + engine_total) * n_annual_flights

    # Print.
    runtime_output.print('Maintenance costs (per year): ' + f"{int(maintenance_costs):,}" + ' €')

    # Prepare output data.
    dict_operating_costs['maintenance_costs'] = {}
    dict_operating_costs['maintenance_costs']['maintenance_costs'] = maintenance_costs

    return dict_operating_costs
