def calculate_direct_operating_costs(dict_ac_data, dict_operating_costs, runtime_output):
    """Calculate direct operating costs.

    // TODO Cost per depreciation period: Check comment below
    The method implemented below is a simplified method based on "From Aircraft Performance to Aircraft Assessment“ by
    J. Thorbeck [1]. The total operating cost are split into direct (DOC) and indirect operating cost (IOC). The DOC are
    directly influenced by the parameters and the performance of the aircraft and are commonly used for aircraft
    evaluation. Two elements are required for the simplified DOC model: Route independent (fixed) and route dependent
    (variable) costs. They are determined for one year and the entire depreciation period.

    This function adds the 'direct_operating_costs' dict to the 'dict_operating_costs' that contains the following
    parameters:
        - array direct_operating_costs_annual: Direct operating costs per year (range dependent) in EUR
        - array direct_operating_costs_depreciation_period: Direct operating costs per depreciation period (range
        dependent) in EUR
        - float direct_operating_costs_annual_design_point: Direct operating costs per year at design point for current
        aircraft in EUR
        - float direct_operating_costs_depreciation_period_design_point: Direct operating costs per depreciation period
        at design point for current aircraft in EUR

    [1] J. Thorbeck: "From Aircraft Performance to Aircraft Assessment“ (YEAR)

    :param dict dict_ac_data: Dict containing parameters and values from aircraft exchange and module configuration file
    :param dict dict_operating_costs: Dict containing parameters from cost estimation calculation
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return dict dict_operating_costs: Dict containing parameters from cost estimation calculation
    """

    # Extract necessary values from 'dict_operating_costs'.
    idx_design_point = dict_operating_costs['payload_range_diagram_data']['idx_design_point']
    route_dependent_costs_annual = dict_operating_costs['route_dependent_costs']['route_dependent_costs_annual']
    route_dependent_costs_depreciation_period = \
        dict_operating_costs['route_dependent_costs']['route_dependent_costs_depreciation_period']
    route_independent_costs_annual = dict_operating_costs['route_independent_costs']['route_independent_costs_annual']
    route_independent_costs_depreciation_period = \
        dict_operating_costs['route_independent_costs']['route_independent_costs_depreciation_period']

    # Calculate.
    direct_operating_costs_annual = route_independent_costs_annual + route_dependent_costs_annual
    direct_operating_costs_depreciation_period = (route_independent_costs_depreciation_period
                                                  + route_dependent_costs_depreciation_period)

    # Print.
    runtime_output.print('DIRECT OPERATING COSTS (per year): '
                         + f"{int(direct_operating_costs_annual[idx_design_point]):,}" + ' €')
    runtime_output.print('----------------------------------------------------------')

    # Prepare output data.
    dict_operating_costs['direct_operating_costs'] = {}
    dict_operating_costs['direct_operating_costs']['direct_operating_costs_annual'] = direct_operating_costs_annual
    dict_operating_costs['direct_operating_costs']['direct_operating_costs_depreciation_period'] = \
        direct_operating_costs_depreciation_period
    dict_operating_costs['direct_operating_costs']['direct_operating_costs_annual_design_point'] = \
        direct_operating_costs_annual[idx_design_point]
    dict_operating_costs['direct_operating_costs']['direct_operating_costs_depreciation_period_design_point'] = \
        direct_operating_costs_depreciation_period[idx_design_point]

    return dict_operating_costs
