/*  Copyright (C) 2012-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.  // NOLINT
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ECOLOGICAL_ASSESSMENT_SRC_STANDARD_STRATEGY_EMISSION_CALCULATION_LIFE_CYCLE_EMISSIONS_LCA_SCHAEFER_SCHAEFERIODATA_H_
#define ECOLOGICAL_ASSESSMENT_SRC_STANDARD_STRATEGY_EMISSION_CALCULATION_LIFE_CYCLE_EMISSIONS_LCA_SCHAEFER_SCHAEFERIODATA_H_

#include <string>
#include <unordered_map>
#include <utility>
#include <vector>
#include <array>
#include <memory>

#include <moduleBasics/runtimeIO.h>
#include <aixml/node.h>
#include <engine/engine.h>

#include "../../../emissionsClass.h"
#include "schaeferConfig.h"

// TODO(franz#1#): T126 check if sum of components = 100%
/** \class SchaeferIOData
 * \brief Class for the settings that are needed in the schaefer emissions strategy.
 */
class SchaeferIOData {
 public:
    const std::string sparePartATA[7] = { /**< ATA chapters which should be exchanged during the operation phase */
        "ATA57", "Fuselage", "ATA55", "ATA55", "ATA32", "ATA54", "PowerUnit"};
    const std::string sparePartName[7] = { /**< Name of the spare part (needs to be the same as main parts right now)*/
        "Wing", "Fuselage", "Tailplane", "Fin", "LandingGear", "Pylons", "PowerUnit"};
    const std::string sparePartATAPowerUnit[3] = { /**< ATA chapter of spare parts for power unit*/
        "ATA28", "ATA36", "ATA54"};//Belong to Engine Maintenance Cost because in PowerUnit RC with in it, attention: ATA54 Pylon+Nacelles -> weight ratio
    const std::string sparePartATAAirframe[4] = { /**< ATA chapter of spare parts for airframe*/
        "ATA51", "ATA52", "ATA53", "ATA56"};//Fuselage Maintenance Cost because in Fuselage RC implicitly included (only ATA not separately mentioned in RC Systems etc.)
    const std::string mainPartsName[7] = {"Wing", "Fuselage", "Tailplane", "Fin", "LandingGear", "Pylons", "PowerUnit"};/**< Names of the aircraft's main parts */
    const std::string mainPartsPath[7] = { /**< Path name of main parts in MassBreakdown*/
        "Wing", "Fuselage", "Empennage/Tailplane", "Empennage/Fin", "LandingGear", "Pylons", "PowerUnit"};
    const std::string mainPartsRCPath[7] = { /**< Path name of mainparts in recurring costs*/
        "Wing", "Fuselage", "Empennage", "Empennage", "LandingGear", "Pylons", "PowerUnit/PowerUnitTotal"};
    const std::string materialName[5] = {"Al", "CFRP", "St", "Ti", "Ni"};/**< Name of used materials*/
    const std::string transportationName[4] = {"Aircraft", "Truck", "Ship", "Train"};/**< Name of the means of transportation*/
    const std::string componentName[6] = {/**< Name of the transported component (if component is also contained in mainPartsName, the name must be the same here)*/
        "Wing", "Fuselage", "Tailplane", "Fin", "LandingGear", "PowerUnit"};
    const std::string componentXMLPath[6] = {/**< XML path of transported component in component_design*/
        "wing", "fuselage", "Empennage/Tailplane", "Empennage/Fin", "landing_gear", "PowerUnit"};
    const std::string componentTransportation[6] = {"Aircraft", "Aircraft", "Aircraft", "Aircraft", "Truck", "Aircraft"};/**< Chosen mean of transportation*/

    SchaeferConfig configuration; /**< Object of class schaeferConfig which contains data from configuration file*/
    Engine myEngine; /**< Object of class engine that contains engine data */
    std::string acftName;/**< Name of the aircraft */

    /** \class mission
     * \brief Class for the emissions of one mission
     */
    class mission {
     public:
        Emissions LTO; /**< LTO emissions of mission [kg] */
        Emissions Cruise; /**< Cruise emissions of mission [kg] */
        /** \brief Constructor of class mission */
        mission() {}
        /** \brief Destructor of class mission */
        virtual ~mission() {}
    };
    mission theTestMission; /**< Object of class mission which contains the test mission emissions */
    mission theStudyMission; /**< Object of class mission which contains the study mission emissions */

    /*Requirements and specifications*/
    double v_max;/**<Maximum operating speed (maximum dynamic pressure)[m/s]*/
    int flightsPerYear;/**< Number of flights per year [-]*/
    int durationOperation; /**<Duration of the operating phase [yr]*/
    double seatsAvailable; /**< Total number of available seats [-] */
    /*Aerodynamics*/
    double S_ref;/**< Reference area of main wing [m^2]*/
    /* Masses */
    double MTOM; /**< Maximum take-off mass [kg] */
    double OME; /**< Operating empty mass [kg] */
    std::vector<double> sparePartsMass;/**< Vector contains masses of spare parts wing, fuselage, stabiliser, fin, landing gear and power unit [kg]*/
    std::vector<double> mainPartsMass;/**< Vector contains masses of main parts wing, fuselage, stabiliser, fin, landing gear, pylon and power unit [kg]*/
    std::vector<double> transportedCompMass;/**< Vector contains masses of transported component wing, fuselage, stabiliser, fin, landing gear and power unit [kg]*/
    double airframeMass; /**< Mass of the airframe components [kg]*/
    double empennageMass; /**< Mass of empennage [kg] */
    double finMassRatio;/**< Ratio of fin mass to empennage mass [-] */
    double stabilizerMassRatio;/**< Ratio stabilizer mass to empennage mass [-] */
    double pylonMassRatio;/**< Ratio pylon mass to pylon+nacelle mass [-]*/
    double nacelleMassRatio;/**< Ratio pylon mass to pylon+nacelle mass [-]*/
    double engineMass; /** Mass of all engines [kg] */
    double nacelleMass;/**< Mass of all nacelles [kg]*/
    double pylonMass;/**< Mass of all pylons [kg]*/
    double bleedAirSystemMass;/**< Mass of overall bleed air system [kg]*/
    double fuelSystemMass;/**< Mass of overall fuel system [kg]*/
    double otherEngineSystemsMass;/**< Mass of other engine system [kg]*/

    /*Propulsion*/
    int numberOfEngines; /**< Number of engines [-] */
    double fuelFlowContThrustCruise; /**< Fuel flow at continuous thrust at cruise for one scaled engine [kg/s]*/

    /*Systems*/
    double propDesignPowerTotal;/**< Propulsion System: Total energy demand [kW]*/

    /*Performance Design Mission*/
    double rangeDesign;/**< Range Design Mission [m] */
    double blocktimeDesign;/**< Block time of design mission [h]*/
    double flightTimeDesign;/**< Flight time of design mission [h]*/
    double taxifuelDesign;/**< Total taxi fuel for design mission [kg]*/
    double tripfuelDesign;/**< Trip fuel of design mission [kg]*/
    double blockfuelDesign;/**< Block fuel of design mission [kg]*/
    double LTOfuelDesign;/**< Fuel mass of design mission for landing and take off [kg]*/
    double taxiTime;/** Total taxi time of design mission [s]*/

    /*Performance Study Mission*/
    double rangeStudy; /**< Range Study Mission [m] */
    double flightTimeStudy;/**< Flight time of study mission [s]*/
    double taxifuelStudy;/**< Taxi fuel of study mission before takeoff [kg]*/
    double tripfuelStudy;/**< Trip fuel of study mission [kg]*/
    double studyMissionPAX;/**< Number of PAX for study mission [-]*/
    double blockfuelStudy;/**< Block fuel of study mission [kg]*/


    /** \brief Construct a new schaefer IO data object
     * \param rtIO Access to rtIO
     * \param ecoAssXML Access to ecological assessment results xml
     */
    explicit SchaeferIOData(std::shared_ptr<RuntimeIO> rtIO, std::shared_ptr<node> ecoAssXML);

    /** \brief Destructor of class SchaeferIOData
     */
    virtual ~SchaeferIOData() {}

    /** \brief Function reads all input data from aircraft exchange file
     * \param rtIO Access to rtIO
     */
    void readAircraft(std::shared_ptr<RuntimeIO> rtIO);

    /** \brief Function reads masses, fills mass vectors and calculate mass ratios
     * \param rtIO Access to rtIO
     */
    void getMasses(std::shared_ptr<RuntimeIO> rtIO);

    /** \brief Function reads masses, fills mass vectors and calculate mass ratios
     * \param rtIO Access to rtIO
     */
    void getMissionFileData(std::shared_ptr<RuntimeIO> rtIO);

    /** \brief Function reads emissions from ecological_assessment_results.xml
     * \param ecologicalAssessmentXML Access to ecological_assessment_results.xml data
     */
    void readEcologicalAssessmentXML(std::shared_ptr<node> ecologicalAssessmentXML);
};

#endif // ECOLOGICAL_ASSESSMENT_SRC_STANDARD_STRATEGY_EMISSION_CALCULATION_LIFE_CYCLE_EMISSIONS_LCA_SCHAEFER_SCHAEFERIODATA_H_
