# Import standard modules.
import math


def cabin_slenderness_ratio_calculation(dict_ac_exchange, dict_mod_config, geometry_description, runtime_output):
    """ Calculates cabin slenderness ratio from given class definition and total cabin width.

        References and calculation method:
            [1] Prof. Dr.-Ing. Juergen Thorbeck TU-Berlin (2006), Script "Flugzeugentwurf I und II" chapter A

    :param dict dict_ac_exchange: Dict containing parameter and according values from aircraft exchange file
    :param dict dict_mod_config: Dict containing parameter and according values from module configuration file
    :param dict geometry_description: Dict containing geometry parameter and values of current fuselage design
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return dict geometry_description: Dict containing geometry parameter and values of current fuselage design
            dict interior_description: Dict containing cabin interior parameter and values of current fuselage design
    """
    ''' initialize local parameter '''
    interior_description = {}
    cabin_length_per_class = []
    # ToDo: ersetzen mit constants from lib
    convert_feet_to_inch = 12

    # read minimum required door space
    minimum_required_door_space = 0
    for exit_type in \
            dict_ac_exchange['dict_cs_requirements']['fuselage_design_cs_requirements']['exit_definition']['exit']:
        minimum_required_door_space = max(minimum_required_door_space, round(float(exit_type['width']['value']), 2))

    # read galley information and set full size trolleys per pax for selected range scenario
    range_scenario = dict_ac_exchange['relevant_classes']['range_scenario']
    full_size_trolley_per_pax = (
        dict_mod_config['pax_per_full_size_trolley_regional_range'] if range_scenario == 'regional'
        else dict_mod_config['pax_per_full_size_trolley_short_range'] if range_scenario == 'short_range'
        else dict_mod_config['pax_per_full_size_trolley_medium_range'] if range_scenario == 'medium_range'
        else dict_mod_config['pax_per_full_size_trolley_long_range']
    )

    # read size of internal bulk area per pax for regional aircraft
    internal_bulk_area = (dict_mod_config['specific_internal_bulk_area']
                          if dict_mod_config['use_internal_bulk_area'] else 0.0)

    i = 0
    while i < len(dict_ac_exchange['relevant_classes']['passenger_classes']['class_names']):
        width_of_seat_rows_per_class = []
        class_name = dict_ac_exchange['relevant_classes']['passenger_classes']['class_names'][i]
        class_distribution = dict_ac_exchange['relevant_classes']['class_percentages'][i]
        seat_abreast = dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['seat_abreast'].split('_')
        total_class_abreast = sum((int(abreast) for abreast in seat_abreast))
        class_seat_pitch = \
            (sum(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['seat_pitch'].values())
             / len(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['seat_pitch']))

        # Estimate the number of seats per row of current payload tube and current passenger class.
        if int(seat_abreast[0]) == 0:
            number_of_inner_armrests = 0
            number_of_outer_armrests = 0
        elif int(seat_abreast[0]) == 1:
            number_of_inner_armrests = 0
            number_of_outer_armrests = 2
        else:
            number_of_inner_armrests = int(seat_abreast[0]) - 1
            number_of_outer_armrests = 2
        # Loop across all entries of "number of seats per row" to estimate seat row widths of current passenger class.
        width_of_seat_rows_per_class.append(
            int(seat_abreast[0])
            * float(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['passenger_seats'][
                        'seat_left_side']['width']['value'])
            + (number_of_inner_armrests
               * float(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['passenger_seats'][
                           'seat_left_side']['width_inner_armrest']['value']))
            + (number_of_outer_armrests
               * float(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['passenger_seats'][
                           'seat_left_side']['width_outer_armrest']['value'])))

        if int(seat_abreast[-1]) == 0:
            number_of_inner_armrests = 0
            number_of_outer_armrests = 0
        elif int(seat_abreast[-1]) == 1:
            number_of_inner_armrests = 0
            number_of_outer_armrests = 2
        else:
            number_of_inner_armrests = int(seat_abreast[-1]) - 1
            number_of_outer_armrests = 2
        width_of_seat_rows_per_class.append(
            int(seat_abreast[-1])
            * float(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['passenger_seats'][
                        'seat_right_side']['width']['value'])
            + (number_of_inner_armrests
               * float(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['passenger_seats'][
                           'seat_right_side']['width_inner_armrest']['value']))
            + (number_of_outer_armrests
               * float(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['passenger_seats'][
                           'seat_right_side']['width_outer_armrest']['value']))
        )
        # Check if the selected fuselage type is a wide body aircraft -> if true: -> append width of center row
        if len(seat_abreast) == 3:
            if int(seat_abreast[1]) == 0:
                number_of_inner_armrests = 0
                number_of_outer_armrests = 0
            elif int(seat_abreast[1]) == 1:
                number_of_inner_armrests = 0
                number_of_outer_armrests = 2
            else:
                number_of_inner_armrests = int(seat_abreast[1]) - 1
                number_of_outer_armrests = 2
            width_of_seat_rows_per_class.append(
                int(seat_abreast[1])
                * float(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['passenger_seats'][
                            'seat_in_the_center']['width']['value'])
                + (number_of_inner_armrests
                   * float(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['passenger_seats'][
                               'seat_in_the_center']['width_inner_armrest']['value']))
                + (number_of_outer_armrests
                   * float(dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['passenger_seats'][
                               'seat_in_the_center']['width_outer_armrest']['value']))
            )
            # reorder the list "width_of_seat_rows_per_class" for wide body aircraft (left_side, center, right_side)
            center = width_of_seat_rows_per_class[-1]
            width_of_seat_rows_per_class[-1] = width_of_seat_rows_per_class[1]
            width_of_seat_rows_per_class[1] = center

        # estimate the average width of class seats
        class_seat_width = round(sum(width_of_seat_rows_per_class) / total_class_abreast, 3)

        # calculate passenger per class by given class distribution
        pax_per_class = math.ceil(class_distribution * dict_ac_exchange['total_number_of_passengers'])

        # calculate number of seat rows per class
        seat_rows_per_class = math.ceil(pax_per_class / total_class_abreast)

        # calculate service area of lavatories per class
        if not dict_mod_config['use_specific_number_of_class_lavatories']['use_specific_number_of_class_lavatories_ID'
                                                                          + str(dict_ac_exchange['relevant_classes'][
                                                                                    'class_indexes'][i])]:
            number_of_class_lavatories = math.ceil(pax_per_class /
                                                   dict_ac_exchange['relevant_classes']['passenger_classes']
                                                   [class_name]['class_lavatory']['pax_per_lavatory'])
        else:
            number_of_class_lavatories = dict_mod_config['specific_number_of_class_lavatories'][
                'specific_number_of_class_lavatories_ID'
                + str(dict_ac_exchange['relevant_classes']['class_indexes'][i])]

        # check if a lavatory is selected for current class -> if true: -> calculate service area of lavatories
        if dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['class_lavatory']['lavatory_name'] \
                is not None:
            class_service_area_lavatory = \
                round(number_of_class_lavatories
                      * float(dict_ac_exchange['relevant_classes']['passenger_classes']
                              [class_name]['class_lavatory']['lavatory']['depth']['value'])
                      * float(dict_ac_exchange['relevant_classes']['passenger_classes']
                              [class_name]['class_lavatory']['lavatory']['width']['value']), 2)
        # else condition: No lavatory is selected for current class -> set service area of lavatories to zero
        else:
            class_service_area_lavatory = 0

        # calculate service area of wardrobes per class
        units_of_wardrobe = 0
        if dict_ac_exchange['relevant_classes']['passenger_classes'][class_name]['class_wardrobe']['use_wardrobe']:
            units_of_wardrobe = \
                (math.ceil(dict_ac_exchange['relevant_classes']['passenger_classes']
                           [class_name]['class_wardrobe']['wardrobe_space'] * pax_per_class
                           / float(dict_ac_exchange['relevant_classes']['passenger_classes']
                                   [class_name]['class_wardrobe']['dimension']['width']['value'])))
            class_service_area_wardrobe = round(units_of_wardrobe *
                                                float(dict_ac_exchange['relevant_classes']['passenger_classes']
                                                      [class_name]['class_wardrobe']['dimension']['width']['value']) *
                                                float(dict_ac_exchange['relevant_classes']['passenger_classes']
                                                      [class_name]['class_wardrobe']['dimension']['depth']['value']), 2)
        else:
            class_service_area_wardrobe = 0

        # calculate service area of galleys per class
        if not dict_mod_config['use_specific_number_of_full_size_trolleys']:
            number_of_full_size_trolley = math.ceil(pax_per_class / full_size_trolley_per_pax)
        else:
            number_of_full_size_trolley = dict_mod_config['specific_number_of_full_size_trolleys']
        class_service_area_galleys = round(number_of_full_size_trolley *
                                           float(dict_ac_exchange['dict_accommodation']['accommodation_definitions']
                                                 ['trolley_definition']['full_size_trolley']['depth']['value']) *
                                           float(dict_ac_exchange['dict_accommodation']['accommodation_definitions']
                                                 ['trolley_definition']['full_size_trolley']['width']['value']), 2)

        # calculate total service area per class and passenger
        class_total_service_area_per_pax = \
            round((class_service_area_lavatory + class_service_area_wardrobe + class_service_area_galleys)
                  / pax_per_class, 2)

        # calculate cabin length per class
        # single-aisle aircraft
        if dict_ac_exchange['fuselage_type'] == "single_aisle":
            cabin_length_per_class.append(round(pax_per_class / total_class_abreast
                                          * (class_seat_pitch
                                             + (class_total_service_area_per_pax / class_seat_width)
                                             + (internal_bulk_area / (dict_mod_config['passenger_aisle_width']
                                                                      / total_class_abreast + class_seat_width))
                                             + minimum_required_door_space
                                             * (total_class_abreast
                                                / pax_per_class + class_seat_pitch
                                                / (float(dict_ac_exchange['dict_cs_requirements']
                                                         ['fuselage_design_cs_requirements']
                                                         ['maximum_allowed_distance_between_exits']['value'])
                                                   * convert_feet_to_inch))), 3))

        # wide-body aircraft
        if dict_ac_exchange['fuselage_type'] == "wide_body":
            cabin_length_per_class.append(round(pax_per_class / total_class_abreast
                                          * (class_seat_pitch
                                             + (class_total_service_area_per_pax / class_seat_width)
                                             + (internal_bulk_area / (dict_mod_config['passenger_aisle_width']
                                                                      / total_class_abreast + class_seat_width))
                                             + 2 * minimum_required_door_space
                                             * (total_class_abreast
                                                / pax_per_class + class_seat_pitch
                                                / (float(dict_ac_exchange['dict_cs_requirements']
                                                         ['fuselage_design_cs_requirements']
                                                         ['maximum_allowed_distance_between_exits']['value'])
                                                   * convert_feet_to_inch))), 3))

        ''' set class dependent interior parameter '''
        number_of_seats_left_side = int(seat_abreast[0]) * seat_rows_per_class
        number_of_seats_right_side = int(seat_abreast[-1]) * seat_rows_per_class
        number_of_seats_in_the_center = 0
        if dict_ac_exchange['fuselage_type'] == "wide_body":
            number_of_seats_in_the_center = int(seat_abreast[1]) * seat_rows_per_class
        interior_description[class_name] = {}
        interior_description[class_name]['seat_abreast'] = seat_abreast
        interior_description[class_name]['pax_per_class'] = pax_per_class
        interior_description[class_name]['units_of_wardrobe'] = units_of_wardrobe
        interior_description[class_name]['seat_rows_per_class'] = seat_rows_per_class
        interior_description[class_name]['cabin_length_per_class'] = cabin_length_per_class
        interior_description[class_name]['number_of_seats_left_side'] = number_of_seats_left_side
        interior_description[class_name]['number_of_seats_right_side'] = number_of_seats_right_side
        interior_description[class_name]['number_of_class_lavatories'] = number_of_class_lavatories
        interior_description[class_name]['number_of_full_size_trolley'] = number_of_full_size_trolley
        interior_description[class_name]['width_of_seat_rows_per_class'] = width_of_seat_rows_per_class
        interior_description[class_name]['number_of_seats_in_the_center'] = number_of_seats_in_the_center

        i += 1

    ''' store total cabin slenderness ratio to geometry description dictionary '''
    geometry_description['cabin_slenderness_ratio'] = \
        (round(geometry_description['total_cabin_width'] / sum(cabin_length_per_class), 4))

    ''' debug print '''
    runtime_output.debug('Debug: The cabin_slenderness_ratio_calculation function was successfully executed!')

    return geometry_description, interior_description
