# Import standard modules.
import os
import sys
import xml.etree.ElementTree

from datapostprocessingmodule import convert_element_tree_to_dictionary
import pycoordinatesystemconversion as py11csc


def prepare_external_data(paths_and_names, routing_dict, dict_ac_exchange, dict_mod_config, runtime_output):
    """  Function prepare external data for project specific calculations.

        :param dict paths_and_names: Dictionary containing system paths and ElementTrees
        :param dict routing_dict: Dictionary containing routing parameter
        :param dict dict_ac_exchange: Dict containing parameter and according values from aircraft exchange file
        :param dict dict_mod_config: Dict containing parameter and according values from module configuration file
        :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
        :return dict dict_ac_exchange: Dict containing parameter and according values from aircraft exchange file
    """
    ''' initialize local parameter '''
    indexes = []
    percentage = []
    seat_details = None
    lavatory_details = {}
    relevant_classes = {}
    relevant_cargo_accommodation = {'uld_type': None}
    class_distribution = dict_ac_exchange['class_distribution'].split('/')
    # Todo ersetzen durch constants from lib
    meter_to_nautical_mile = 1852

    ''' Import passenger and cargo accommodation file '''
    # Initialize element tree with content of file and return root element (if given).
    try:
        # Attempt to create an ElementTree and get the root element from the XML file.
        accommodation_tree = \
            xml.etree.ElementTree.ElementTree(file=paths_and_names['working_directory'] 
                                              + '/fuselage_design_lib/accommodation_definitions.xml')
        dict_ac_exchange['dict_accommodation'] = convert_element_tree_to_dictionary(accommodation_tree.getroot())
    # Exception handling for operating system (OS) error.
    except OSError:
        # Handle an error if the XML file cannot be opened. Print an error message and log it to a log file.
        runtime_output.critical('Error: The passenger and cargo accommodation definition xml-file could not be opened '
                                'or found. \n                                     Program aborted!')
        sys.exit('Accommodation definition file could not be opened.')

    ''' Import fuselage design cs requirements file '''
    # Initialize element tree with content of file and return root element (if given).
    try:
        # Attempt to create an ElementTree and get the root element from the XML file.
        cs_requirement_tree = \
            xml.etree.ElementTree.ElementTree(file=paths_and_names['working_directory'] 
                                              +'/fuselage_design_lib/fuselage_design_cs_requirements.xml')
        dict_ac_exchange['dict_cs_requirements'] = convert_element_tree_to_dictionary(cs_requirement_tree.getroot())
    # Exception handling for operating system (OS) error.
    except OSError:
        # Handle an error if the XML file cannot be opened. Print an error message and log it to a log file.
        runtime_output.critical('Error: The fuselage design cs requirements xml-file could not be opened or found. \n'
                                '                                     Program aborted!')
        sys.exit('Fuselage design cs requirements file could not be opened.')

    runtime_output.print('Import of fuselage design cs requirements xml-file and accommodation definition xml-file '
                         'successfully.')

    ''' Transform local position of input parameter into global positions '''
    # Get id of aerodynamic surface with the name == 'main_wing'.
    index = 0
    for _, value in dict_ac_exchange['aerodynamic_surface_name'].items():
        if value == 'main_wing':
            break
        index += 1

    ''' Extract project specific passenger and cargo accommodation '''
    # Find relevant passenger classes.
    i = 0
    for passenger_class in class_distribution:
        if not passenger_class == '0':
            indexes.append(i)
            percentage.append(round(float(passenger_class), 3))
        i += 1
    relevant_classes['class_indexes'] = indexes
    relevant_classes['class_percentages'] = percentage

    if sum(percentage) > 1 or sum(percentage) <= 0:
        runtime_output.critical('Error: The given class distribution is greater than 1 (100%). '
                                'Please change class distribution in the design specification. \n'
                                '                                     Program aborted!')
        sys.exit('Selected class distribution not possible.')

    # Get relevant class names and associated accommodation items.
    relevant_classes['passenger_classes'] = {}
    relevant_classes['passenger_classes']['class_names'] = []
    class_names = list(dict_mod_config['passenger_class_name'].values())
    for index in indexes:
        relevant_classes['passenger_classes'][class_names[index]] = {}
        relevant_classes['passenger_classes']['class_names'].append(class_names[index])
        # read seat abreast
        relevant_classes['passenger_classes'][class_names[index]]['seat_abreast'] = \
            dict_mod_config['passenger_class_abreast']['passenger_class_abreast_ID' + str(index)]
        if relevant_classes['passenger_classes'][class_names[index]]['seat_abreast'] is None:
            runtime_output.critical('Error: At least one selected class has no entered number of seat abreast.\n'
                                    '                                     Program aborted!')
            sys.exit('Missing number of seat abreast for ' + class_names[index] + ' class.')

        # read seat name
        relevant_classes['passenger_classes'][class_names[index]]['seat_name'] = {}
        relevant_classes['passenger_classes'][class_names[index]]['seat_name']['seat_name_left_side'] = \
            dict_mod_config['passenger_seat_on_left_side_name']['passenger_seat_on_left_side_name_ID' + str(index)]
        relevant_classes['passenger_classes'][class_names[index]]['seat_name']['seat_name_right_side'] = \
            dict_mod_config['passenger_seat_on_right_side_name']['passenger_seat_on_right_side_name_ID' + str(index)]
        if relevant_classes['passenger_classes'][class_names[index]]['seat_name']['seat_name_left_side'] is None or \
                relevant_classes['passenger_classes'][class_names[index]]['seat_name']['seat_name_right_side'] is None:
            runtime_output.critical('Error: At least one selected class has no entered seat name.\n'
                                    '                                     Program aborted!')
            sys.exit('Missing name of passenger seat for ' + class_names[index] + ' class.')

        # read seat data from accommodation dictionary
        relevant_classes['passenger_classes'][class_names[index]]['passenger_seats'] = {}
        # seats on the left side of the aircraft
        for seat in dict_ac_exchange['dict_accommodation']['accommodation_definitions']['passenger_seats']['seat']:
            if seat['name']['value'] == \
                    relevant_classes['passenger_classes'][class_names[index]]['seat_name']['seat_name_left_side']:
                seat_details = seat
                break

        if seat_details is None:
            runtime_output.critical('Error: The selected class seat for the ' + class_names[index]
                                    + ' class does not exist in accommodation definition file.\n'
                                      '                                     Program aborted!')
            sys.exit('Selected seat does not exist in accommodation file.')
        else:
            relevant_classes['passenger_classes'][class_names[index]]['passenger_seats']['seat_left_side'] \
                = seat_details

        # seats on the right side of the aircraft
        for seat in dict_ac_exchange['dict_accommodation']['accommodation_definitions']['passenger_seats']['seat']:
            if seat['name']['value'] == \
                    relevant_classes['passenger_classes'][class_names[index]]['seat_name']['seat_name_right_side']:
                seat_details = seat
                break

        if seat_details is None:
            runtime_output.critical('Error: The selected class seat for the ' + class_names[index]
                                    + ' class does not exist in accommodation definition file.\n'
                                      '                                     Program aborted!')
            sys.exit('Selected seat does not exist in accommodation file.')
        else:
            relevant_classes['passenger_classes'][class_names[index]]['passenger_seats']['seat_right_side'] \
                = seat_details

        # read seat pitch
        relevant_classes['passenger_classes'][class_names[index]]['seat_pitch'] = {}
        relevant_classes['passenger_classes'][class_names[index]]['seat_pitch']['seat_pitch_left_side'] = \
            dict_mod_config['passenger_seat_on_left_side_pitch']['passenger_seat_on_left_side_pitch_ID' + str(index)]
        relevant_classes['passenger_classes'][class_names[index]]['seat_pitch']['seat_pitch_right_side'] = \
            dict_mod_config['passenger_seat_on_right_side_pitch']['passenger_seat_on_right_side_pitch_ID' + str(index)]

        # read outer seat space
        relevant_classes['passenger_classes'][class_names[index]]['seat_space'] = \
            dict_mod_config['passenger_seat_space']['passenger_seat_space_ID' + str(index)]

        # read class associated lavatories
        relevant_classes['passenger_classes'][class_names[index]]['class_lavatory'] = {}
        relevant_classes['passenger_classes'][class_names[index]]['class_lavatory']['lavatory_name'] = \
            dict_mod_config['passenger_lavatory_name']['passenger_lavatory_name_ID' + str(index)]
        relevant_classes['passenger_classes'][class_names[index]]['class_lavatory']['pax_per_lavatory'] = \
            dict_mod_config['passenger_per_lavatory']['passenger_per_lavatory_ID' + str(index)]
        relevant_classes['passenger_classes'][class_names[index]]['class_lavatory']['use_specific_lavatory_number'] = \
            dict_mod_config['use_specific_number_of_class_lavatories']['use_specific_number_of_class_lavatories_ID'
                                                                       + str(index)]
        relevant_classes['passenger_classes'][class_names[index]]['class_lavatory']['specific_lavatory_number'] = \
            dict_mod_config['specific_number_of_class_lavatories']['specific_number_of_class_lavatories_ID'
                                                                   + str(index)]

        # read class associated lavatory from accommodation definition file
        for lavatory in dict_ac_exchange['dict_accommodation']['accommodation_definitions']['lavatories']['lavatory']:
            if lavatory['name']['value'] == \
                    relevant_classes['passenger_classes'][class_names[index]]['class_lavatory']['lavatory_name']:
                lavatory_details = lavatory
                break

        if lavatory_details is None:
            runtime_output.critical('Error: The selected class lavatory for the ' + class_names[index]
                                    + ' class does not exist in accommodation definition file.\n'
                                      '                                     Program aborted!')
        else:
            relevant_classes['passenger_classes'][class_names[index]]['class_lavatory']['lavatory'] \
                = lavatory_details

        # read class associated wardrobe parameter
        relevant_classes['passenger_classes'][class_names[index]]['class_wardrobe'] = {}
        relevant_classes['passenger_classes'][class_names[index]]['class_wardrobe']['use_wardrobe'] = \
            dict_mod_config['use_wardrobe_for_passenger_class']['use_wardrobe_for_passenger_class_ID' + str(index)]
        if relevant_classes['passenger_classes'][class_names[index]]['class_wardrobe']['use_wardrobe']:
            relevant_classes['passenger_classes'][class_names[index]]['class_wardrobe']['wardrobe_space'] = \
                dict_mod_config['wardrobe_space_per_passenger']['wardrobe_space_per_passenger_ID' + str(index)]
            relevant_classes['passenger_classes'][class_names[index]]['class_wardrobe']['dimension'] =\
                dict_ac_exchange['dict_accommodation']['accommodation_definitions']['wardrobe']

        # Check if the current project is a wide body aircraft
        #  -> if true: -> set wide body relevant passenger accommodation.
        if dict_ac_exchange['fuselage_type'] == 'wide_body':
            # read seat name
            relevant_classes['passenger_classes'][class_names[index]]['seat_name']['seat_name_in_the_center'] = \
                dict_mod_config['passenger_seat_in_the_center_name'][
                    'passenger_seat_in_the_center_name_ID' + str(index)]

            if relevant_classes['passenger_classes'][class_names[index]]['seat_name']['seat_name_in_the_center'] is None:  # noPep8 e501
                runtime_output.critical('Error: At least one selected class has no entered seat name.\n'
                                        '                                     Program aborted!')
                sys.exit('Missing name of passenger seat for ' + class_names[index] + ' class.')

            # seats on the left side of the aircraft
            for seat in dict_ac_exchange['dict_accommodation']['accommodation_definitions']['passenger_seats']['seat']:
                if seat['name']['value'] == \
                        relevant_classes['passenger_classes'][class_names[index]]['seat_name'][
                            'seat_name_in_the_center']:
                    seat_details = seat
                    break

            if seat_details is None:
                runtime_output.critical('Error: The selected class seat for the ' + class_names[index]
                                        + ' class does not exist in accommodation definition file.\n'
                                          '                                     Program aborted!')
                sys.exit('Selected seat does not exist in accommodation file.')
            else:
                relevant_classes['passenger_classes'][class_names[index]]['passenger_seats']['seat_in_the_center'] \
                    = seat_details

            # read seat pitch
            relevant_classes['passenger_classes'][class_names[index]]['seat_pitch']['seat_pitch_in_the_center'] = \
                dict_mod_config['passenger_seat_in_the_center_pitch']['passenger_seat_in_the_center_pitch_ID'
                                                                      + str(index)]

    ''' set design range associated scenario '''
    range_in_nautical_mile = dict_ac_exchange['design_mission_range'] / meter_to_nautical_mile
    relevant_classes['range_scenario'] = (
        'regional' if 0 <= range_in_nautical_mile < 500
        else 'short_range' if 500 <= range_in_nautical_mile < 3000
        else 'medium_range' if 3000 <= range_in_nautical_mile < 5500
        else 'long_range')

    ''' set relevant cargo accommodation '''
    if dict_mod_config['uld_type'] == 'container':
        for container in dict_ac_exchange['dict_accommodation']['accommodation_definitions']['unit_load_devices'][
                                          'ul_container']['container']:
            if container['name']['value'] == dict_mod_config['container_name']:
                relevant_cargo_accommodation['uld_type'] = {container['name']['value']: {}}
                relevant_cargo_accommodation['uld_type'][container['name']['value']] = container

        if relevant_cargo_accommodation.get('uld_type') is None:
            runtime_output.critical('Error: The selected uld container type: ' + dict_mod_config['container_name']
                                    + ' is not valid.\n                                     Program aborted!')
            sys.exit('Uld container type not valid.')

    elif dict_mod_config['uld_type'] == 'pallet':
        for pallet in dict_ac_exchange['dict_accommodation']['accommodation_definitions']['unit_load_devices'][
                                          'cargo_pallet']['pallet']:
            if pallet['name']['value'] == dict_mod_config['pallet_name']:
                relevant_cargo_accommodation['uld_type'] = {pallet['name']['value']: {}}
                relevant_cargo_accommodation['uld_type'][pallet['name']['value']] = pallet

        if relevant_cargo_accommodation.get('uld_type') is None:
            runtime_output.critical('Error: The selected cargo pallet type: ' + dict_mod_config['pallet_name']
                                    + ' is not valid.\n                                     Program aborted!')
            sys.exit('Cargo pallet not valid.')

    elif dict_mod_config['uld_type'] == 'mixed':
        for container in dict_ac_exchange['dict_accommodation']['accommodation_definitions']['unit_load_devices'][
                                          'ul_container']['container']:
            if container['name']['value'] == dict_mod_config['container_name']:
                relevant_cargo_accommodation['uld_type'] = {container['name']['value']: {}}
                relevant_cargo_accommodation['uld_type'][container['name']['value']] = container

        if relevant_cargo_accommodation.get('uld_type') is None:
            runtime_output.critical('Error: The selected uld container type: ' + dict_mod_config['container_name']
                                    + ' is not valid.\n                                     Program aborted!')
            sys.exit('Uld container type not valid.')

        else:
            for pallet in dict_ac_exchange['dict_accommodation']['accommodation_definitions']['unit_load_devices'][
                                           'cargo_pallet']['pallet']:
                if pallet['name']['value'] == dict_mod_config['pallet_name']:
                    relevant_cargo_accommodation['uld_type'][pallet['name']['value']] = {}
                    relevant_cargo_accommodation['uld_type'][pallet['name']['value']] = pallet

            if len(relevant_cargo_accommodation.get('uld_type')) < 2:
                runtime_output.critical('Error: The selected cargo pallet type: ' + dict_mod_config['pallet_name']
                                        + ' is not valid.\n                                     Program aborted!')
                sys.exit('Cargo pallet not valid.')
    else:
        runtime_output.critical('Error: The selected uld type: ' + dict_mod_config['uld_type'] + ' is not valid.\n'
                                '                                     Program aborted!')
        sys.exit('Selected uld type not valid.')

    ''' store relevant data to aircraft exchange dictionary '''
    dict_ac_exchange['relevant_classes'] = relevant_classes
    dict_ac_exchange['relevant_cargo_accommodation'] = relevant_cargo_accommodation

    ''' debug print '''
    runtime_output.debug('Debug: The prepare_external_data function was successfully executed!')
    
    dict_ac_exchange = coordinate_system_conversion(dict_ac_exchange)

    return dict_ac_exchange


def coordinate_system_conversion(dict_ac_exchange):
    for key in dict_ac_exchange.keys():
        if "x" in key.split("_"):
            if key.split("_")[-1] != "x":
                if dict_ac_exchange[key.split("_x")[0] + "_x" + key.split("_x")[-1]] is not None:
                    element_3d = py11csc.Element3D(dict_ac_exchange[key.split("_x")[0] + "_x" + key.split("_x")[-1]],
                                                   dict_ac_exchange[key.split("_x")[0] + "_y" + key.split("_x")[-1]],
                                                   dict_ac_exchange[key.split("_x")[0] + "_z" + key.split("_x")[-1]])
                    element_3d.CGAL2AC()
                    dict_ac_exchange[key.split("_x")[0] + "_x" + key.split("_x")[-1]] = element_3d.x()
                    dict_ac_exchange[key.split("_x")[0] + "_y" + key.split("_x")[-1]] = element_3d.y()
                    dict_ac_exchange[key.split("_x")[0] + "_z" + key.split("_x")[-1]] = element_3d.z()
            elif isinstance(dict_ac_exchange[key.split("_x")[0] + "_x"], dict):
                for key in dict_ac_exchange[key.split("_x")[0] + "_x"].keys():
                    if "x" in key.split("_"):
                        if key.split("_")[-1] != "x":
                            if dict_ac_exchange[key.split("_x")[0] + "_x"][key.split("_x")[0]
                                                                           + "_x" + key.split("_x")[-1]] is not None:
                                element_3d = py11csc.Element3D(dict_ac_exchange[key.split("_x")[0] + "_x"][
                                                                  key.split("_x")[0] + "_x" + key.split("_x")[-1]],
                                                               dict_ac_exchange[key.split("_x")[0] + "_y"][
                                                                  key.split("_x")[0] + "_y" + key.split("_x")[-1]],
                                                               dict_ac_exchange[key.split("_x")[0] + "_z"][
                                                                  key.split("_x")[0] + "_z" + key.split("_x")[-1]])
                                element_3d.CGAL2AC()
                                dict_ac_exchange[key.split("_x")[0] + "_x"][
                                    key.split("_x")[0] + "_x" + key.split("_x")[-1]] = element_3d.x()
                                dict_ac_exchange[key.split("_x")[0] + "_y"][
                                    key.split("_x")[0] + "_y" + key.split("_x")[-1]] = element_3d.y()
                                dict_ac_exchange[key.split("_x")[0] + "_z"][
                                    key.split("_x")[0] + "_z" + key.split("_x")[-1]] = element_3d.z()
                        else:
                            if dict_ac_exchange[key.split("_x")[0] + "_x"][key.split("_x")[0] + "_x"] is not None:
                                element_3d = py11csc.Element3D(
                                    dict_ac_exchange[key.split("_x")[0] + "_x"][key.split("_x")[0] + "_x"],
                                    dict_ac_exchange[key.split("_x")[0] + "_y"][key.split("_x")[0] + "_y"],
                                    dict_ac_exchange[key.split("_x")[0] + "_z"][key.split("_x")[0] + "_z"])
                                element_3d.CGAL2AC()
                                dict_ac_exchange[key.split("_x")[0] + "_x"][key.split("_x")[0] + "_x"] = element_3d.x()
                                dict_ac_exchange[key.split("_x")[0] + "_y"][key.split("_x")[0] + "_y"] = element_3d.y()
                                dict_ac_exchange[key.split("_x")[0] + "_z"][key.split("_x")[0] + "_z"] = element_3d.z()
            else:
                if dict_ac_exchange[key.split("_x")[0] + "_x"] is not None:
                    element_3d = py11csc.Element3D(dict_ac_exchange[key.split("_x")[0] + "_x"],
                                                   dict_ac_exchange[key.split("_x")[0] + "_y"],
                                                   dict_ac_exchange[key.split("_x")[0] + "_z"])
                    element_3d.CGAL2AC()
                    dict_ac_exchange[key.split("_x")[0] + "_x"] = element_3d.x()
                    dict_ac_exchange[key.split("_x")[0] + "_y"] = element_3d.y()
                    dict_ac_exchange[key.split("_x")[0] + "_z"] = element_3d.z()

    return dict_ac_exchange
