# Import standard modules.
import os
import sys
import math
import xml.etree.ElementTree as ET


def set_existing_geometry(dict_ac_exchange, dict_mod_config, root_of_existing_geometry_tree, runtime_output):
    """ Set existing geometry parameter for the landing gear design module to use a landing gear geometry from an
         external file.

        :param dict dict_ac_exchange: D Dict containing parameter and according values from aircraft exchange file
        :param dict dict_mod_config: Dict containing parameter and according values from module configuration file
        :param root root_of_existing_geometry_tree: The root node contains all existing geometry data.
        :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
        :return dict landing_gear_description: Dict containing parameter and values of current landing gear estimation
    """
    ''' initialize local parameter '''
    tire_list = []
    lever_arms = {}
    nose_gear_tires = {}
    main_gear_tires = {}
    landing_gear_loads = {}
    fuselage_parameter = {}
    nose_gear_tire_dict = {}
    main_gear_tire_dict = {}
    nose_gear_arrangement = {}
    main_gear_arrangement = {}
    landing_gear_positions = {}
    landing_gear_description = {'landing_gear_positions': {},
                                'nose_gear_description': {},
                                'main_gear_description': {},
                                'tail_tipping_point': {},
                                'landing_gear_limitations': {},
                                'landing_gear_loads': {},
                                'fuselage_parameter': {},
                                'lever_arms': {}
                                }
    convert_inch_to_meter = 0.0254
    minimum_required_wheel_base = 0.0
    gravitational_acceleration = 9.80665
    assembly_names = root_of_existing_geometry_tree.findall(
        './landing_gear/specific/geometry/landing_gear_assembly/name/value')
    tire_names = \
        root_of_existing_geometry_tree.findall(
            './landing_gear/specific/geometry/landing_gear_assembly/assembly_components/'
            'tire_description/name/value')

    ''' set main and nose gear tires '''
    if tire_names is not None:
        # Generate a list of all tire names of the existing geometry.
        for tire in tire_names:
            tire_list.append(tire.text)

        # try to open easa etso tire list
        for index, assembly_name in enumerate(assembly_names):
            if 'nose_gear' in root_of_existing_geometry_tree.find(
                    './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(index) + '"]/name/value').text:
                nose_gear_tire_dict['name'] = tire_names[index].text
                break
        for index, assembly_name in enumerate(assembly_names):
            if 'main_gear' in root_of_existing_geometry_tree.find(
                    './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(index) + '"]/name/value').text:
                main_gear_tire_dict['name'] = tire_names[len(tire_list) - 1].text
                break

        if nose_gear_tire_dict is None:
            runtime_output.critical('Error: No nose gear tires are defined in the given existing geometry. \n'
                                    '                                            Program aborted!')
            sys.exit('Exit information: Missing required landing gear assembly parameter in the existing landing gear '
                     'geometry!')

        if main_gear_tire_dict is None:
            runtime_output.critical('Error: No main gear tires are defined in the given existing geometry. \n'
                                    '                                            Program aborted!')
            sys.exit(
                'Exit information: Missing required landing gear assembly parameter in the existing landing gear '
                'geometry!')

        # Try to open easa etso tire list from xml file.
        try:
            path = os.getcwd() + '/landing_gear_lib/EASA_ETSO_tire_list_2022.xml'
            xml_tree = ET.ElementTree(file=path)
            root_of_aircraft_tire_tree = xml_tree.getroot()

            # read nose gear tire data from easa etso tire list
            for tire in root_of_aircraft_tire_tree.findall('./nose_gear_tire/tire'):
                current_part_number = tire.find('part_number')
                if current_part_number is not None and current_part_number.text == nose_gear_tire_dict['name']:
                    nose_gear_tires = {nose_gear_tire_dict['name']: {child.tag: child.text for child in tire}}
                    del nose_gear_tires[nose_gear_tire_dict['name']]['part_number']
                    break

            # read main gear tire data from easa etso tire list
            for tire in root_of_aircraft_tire_tree.findall('./main_gear_tire/tire'):
                current_part_number = tire.find('part_number')
                if current_part_number is not None and current_part_number.text == main_gear_tire_dict['name']:
                    main_gear_tires = {main_gear_tire_dict['name']: {child.tag: child.text for child in tire}}
                    del main_gear_tires[main_gear_tire_dict['name']]['part_number']
                    break

        # Exception handling for file reading.
        except OSError:
            runtime_output.critical('Error: The file: "EASA_ETSO_tire_list_2022.xml" could not found or opened. \n'
                                    '                                            Program aborted!')
            sys.exit('Read aircraft tire xml is failed for landing gear design!')

        # Set landing gear arrangement.
        nose_gear_count = 0
        main_gear_count = 0
        for index, assembly_name in enumerate(assembly_names):
            if 'nose_gear' in root_of_existing_geometry_tree.find(
                    './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(index) + '"]/name/value').text:
                nose_gear_count += 1
            if 'main_gear' in root_of_existing_geometry_tree.find(
                    './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(index) + '"]/name/value').text:
                main_gear_count += 1

        # Nose gear arrangement and positions
        landing_gear_positions['nose_gear_x_position'] = round(float(root_of_existing_geometry_tree.find(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(0)
            + '"]/position/x/value').text), 4)
        landing_gear_positions['nose_gear_strut_y_position'] = abs(round(float(root_of_existing_geometry_tree.find(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(0)
            + '"]/position/y/value').text), 4))
        landing_gear_positions['nose_gear_z_position'] = round(float(root_of_existing_geometry_tree.find(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(0)
            + '"]/position/z/value').text), 4)
        nose_gear_arrangement['nose_gear_tire_design_load'] = None
        nose_gear_arrangement['number_of_nose_gear_struts'] = nose_gear_count
        nose_gear_arrangement['number_of_nose_gear_axis_per_strut'] = 1
        nose_gear_arrangement['number_of_nose_gear_tires_per_strut'] = len(root_of_existing_geometry_tree.findall(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(0)
            + '"]/assembly_components/tire_description'))
        nose_gear_arrangement['nose_gear_strut_length'] = round(float(root_of_existing_geometry_tree.find(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(0)
            + '"]/assembly_components/strut_length/value').text), 4)

        # Main gear arrangement and positions.
        if main_gear_count < 2:
            runtime_output.critical('Error: Not enough main gear assemblies are defined in the given existing geometry.'
                                    ' \n                                            Program aborted!')
            sys.exit(
                'Exit information: Missing required landing gear assembly parameter in the existing landing gear '
                'geometry!')

        landing_gear_positions['main_gear_x_position'] = round(float(root_of_existing_geometry_tree.find(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(nose_gear_count)
            + '"]/position/x/value').text), 4)
        landing_gear_positions['main_gear_outer_strut_y_position'] = abs(round(float(root_of_existing_geometry_tree.find(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(nose_gear_count)
            + '"]/position/y/value').text), 4))
        landing_gear_positions['main_gear_z_position_outer_strut'] = round(float(root_of_existing_geometry_tree.find(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(nose_gear_count)
            + '"]/position/z/value').text), 4)

        number_of_main_gear_inner_struts = 0
        if main_gear_count > 2:
            number_of_main_gear_inner_struts = main_gear_count - 2
            landing_gear_positions['main_gear_inner_strut_y_position'] = \
                abs(round(float(root_of_existing_geometry_tree.find(
                    './landing_gear/specific/geometry/landing_gear_assembly[@ID="'
                    + str(nose_gear_count + main_gear_count - 1) + '"]/position/y/value').text), 4))
            landing_gear_positions['main_gear_z_position_inner_strut'] = \
                round(float(root_of_existing_geometry_tree.find(
                    './landing_gear/specific/geometry/landing_gear_assembly[@ID="'
                    + str(nose_gear_count + main_gear_count - 1) + '"]/position/z/value').text), 4)

        main_gear_arrangement['main_gear_tire_design_load'] = None
        main_gear_arrangement['number_of_main_gear_struts'] = main_gear_count
        main_gear_arrangement['number_of_main_gear_outer_struts'] = 2
        number_of_nose_gear_tires_per_outer_strut = len(root_of_existing_geometry_tree.findall(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(nose_gear_count)
            + '"]/assembly_components/tire_description'))
        main_gear_arrangement['number_of_main_gear_axis_per_outer_strut'] = \
            math.ceil(number_of_nose_gear_tires_per_outer_strut / 2)
        main_gear_arrangement['number_of_main_gear_tires_per_outer_strut'] = number_of_nose_gear_tires_per_outer_strut
        main_gear_arrangement['main_gear_outer_strut_length'] = round(float(root_of_existing_geometry_tree.find(
            './landing_gear/specific/geometry/landing_gear_assembly[@ID="' + str(nose_gear_count)
            + '"]/assembly_components/strut_length/value').text), 4)
        if main_gear_count > 2:
            main_gear_arrangement['number_of_main_gear_inner_struts'] = number_of_main_gear_inner_struts
            number_of_nose_gear_tires_per_inner_strut = len(root_of_existing_geometry_tree.findall(
                './landing_gear/specific/geometry/landing_gear_assembly[@ID="'
                + str(nose_gear_count + main_gear_count - 1) + '"]/assembly_components/tire_description'))
            main_gear_arrangement['number_of_main_gear_axis_per_inner_strut'] = \
                math.ceil(number_of_nose_gear_tires_per_inner_strut / 2)
            main_gear_arrangement['number_of_main_gear_tires_per_inner_strut'] = \
                number_of_nose_gear_tires_per_inner_strut
            main_gear_arrangement['main_gear_inner_strut_length'] = round(float(root_of_existing_geometry_tree.find(
                './landing_gear/specific/geometry/landing_gear_assembly[@ID="'
                + str(nose_gear_count + main_gear_count - 1) + '"]/assembly_components/strut_length/value').text), 4)

    else:
        runtime_output.critical('Error: No tires are defined in the given existing geometry. \n'
                                '                                            Program aborted!')
        sys.exit('Exit information: Missing required landing gear assembly parameter in the existing landing gear '
                 'geometry!')

    ''' estimate local tire coordinates '''
    if main_gear_arrangement['number_of_main_gear_axis_per_outer_strut'] > 1:
        max_outside_diameter_in_inch = \
            float(main_gear_tires[main_gear_tire_dict['name']]['max_outside_diameter_in_inch'])
        if dict_mod_config['minimum_required_wheel_base'] < (1.10 * max_outside_diameter_in_inch
                                                             * convert_inch_to_meter):
            minimum_required_wheel_base = round(1.10 * (max_outside_diameter_in_inch * convert_inch_to_meter), 4)

        else:
            minimum_required_wheel_base = round(dict_mod_config['minimum_required_wheel_base'], 4)
    # wheel base
    if main_gear_arrangement['number_of_main_gear_axis_per_outer_strut'] == 1:
        main_gear_arrangement['local_tire_position_in_x_for_acn'] = [minimum_required_wheel_base,
                                                                     minimum_required_wheel_base]
    elif main_gear_arrangement['number_of_main_gear_axis_per_outer_strut'] == 2:
        main_gear_arrangement['local_tire_position_in_x_for_acn'] = [0.0,
                                                                     0.0,
                                                                     -minimum_required_wheel_base,
                                                                     -minimum_required_wheel_base]
    elif main_gear_arrangement['number_of_main_gear_axis_per_outer_strut'] == 3:
        main_gear_arrangement['local_tire_position_in_x_for_acn'] = [0.0,
                                                                     0.0,
                                                                     -minimum_required_wheel_base / 2,
                                                                     -minimum_required_wheel_base / 2,
                                                                     -minimum_required_wheel_base,
                                                                     -minimum_required_wheel_base
                                                                     ]

    main_gear_arrangement['wheel_base'] = minimum_required_wheel_base

    # wheel track
    max_section_width_in_meter = round(float(main_gear_tires[main_gear_tire_dict['name']]['max_section_width_in_inch'])
                                       * convert_inch_to_meter, 4)

    if dict_ac_exchange['fuselage_type'] == 'single_aisle':
        wheel_track = round(2 * max_section_width_in_meter * 1.1, 4)
    else:
        wheel_track = round(2.75 * max_section_width_in_meter * 1.1, 4)

    if wheel_track < dict_mod_config['minimum_required_wheel_track']:
        wheel_track = round(float(dict_mod_config['minimum_required_wheel_track']), 4)

    main_gear_arrangement['wheel_track'] = wheel_track
    if main_gear_arrangement['number_of_main_gear_axis_per_outer_strut'] == 1:
        main_gear_arrangement['local_tire_position_in_y_for_acn'] = [-wheel_track / 2, wheel_track / 2]
    if main_gear_arrangement['number_of_main_gear_axis_per_outer_strut'] == 2:
        main_gear_arrangement['local_tire_position_in_y_for_acn'] = [-wheel_track / 2, wheel_track / 2,
                                                                     -wheel_track / 2, wheel_track / 2]
    if main_gear_arrangement['number_of_main_gear_axis_per_outer_strut'] == 3:
        main_gear_arrangement['local_tire_position_in_y_for_acn'] = [-wheel_track / 2, wheel_track / 2,
                                                                     -wheel_track / 2, wheel_track / 2,
                                                                     -wheel_track / 2, wheel_track / 2]

    ''' set maximum rotating angle '''
    if dict_ac_exchange['maximum_angle_off_attack_landing'] is not None:
        maximum_rotation_angle_landing = max(dict_ac_exchange['maximum_angle_off_attack_landing'],
                                             dict_mod_config['tail_strike_limit'])
    else:
        maximum_rotation_angle_landing = dict_mod_config['tail_strike_limit']

    ''' estimate maximum ramp weight '''
    # Check if the maximum take of mass and the taxi fuel mass for take-off are zero.
    #  -> if true: -> Check which kind of fuselage type is selected and set specific starting values.
    if dict_ac_exchange['maximum_take_off_mass'] is None and dict_ac_exchange['taxi_fuel_take_off'] is None:
        # Check if the fuselage type of current project is single aisle
        #  -> if true: -> Set starting value for maximum ramp weight for single aisle aircraft to 70.000 * g0.
        if dict_ac_exchange['fuselage_type'] == 'single_aisle':
            landing_gear_loads['maximum_ramp_weight'] = gravitational_acceleration * 70000
        # Else condition: The fuselage type of current project is weight body.
        #  -> Set starting value for maximum ramp weight for weight body aircraft to 250.000 * g0.
        else:
            landing_gear_loads['maximum_ramp_weight'] = gravitational_acceleration * 250000
    # Else if condition: Check if the taxi fuel take of mass is zero
    #  -> if true: -> Add 1000 kg to the maximum take of mass.
    elif dict_ac_exchange['taxi_fuel_take_off'] is None:
        landing_gear_loads['maximum_ramp_weight'] = gravitational_acceleration \
                                                    * (dict_ac_exchange['maximum_take_off_mass'] + 1000)
    # Else condition: Maximum take-off mass and taxi fuel mass for take-off exists -> estimate maximum ramp weight
    else:
        landing_gear_loads['maximum_ramp_weight'] = gravitational_acceleration \
                                                    * (dict_ac_exchange['maximum_take_off_mass']
                                                       + dict_ac_exchange['taxi_fuel_take_off'])

    ''' set fuselage parameter '''
    # Convert values of both dictionaries to lists
    section_upper_height_list = list(dict_ac_exchange['fuselage_section_upper_height'].values())
    section_lower_height_list = list(dict_ac_exchange['fuselage_section_lower_height'].values())

    # Add corresponding elements of the lists
    section_total_height_list = [x + y if x is not None and y is not None else None for x, y in
                                 zip(section_upper_height_list, section_lower_height_list)]

    # Create a new dictionary with keys from the first dictionary
    result_dict = {key: value for key, value in zip(dict_ac_exchange['fuselage_section_upper_height'].keys(),
                                                    section_total_height_list)}

    # total fuselage section heights
    dict_ac_exchange['fuselage_section_total_height'] = \
        {key.replace('upper', 'total'): value for key, value in result_dict.items()}

    # Set fuselage height
    fuselage_height_key, fuselage_height_value = \
        next(iter(dict_ac_exchange['fuselage_section_total_height'].items()), (None, None))
    if fuselage_height_value is None and dict_ac_exchange['fuselage_type'] == 'single_aisle':
        fuselage_parameter['fuselage_height'] = 3.8
    elif fuselage_height_value is None and dict_ac_exchange['fuselage_type'] == 'wide_body':
        fuselage_parameter['fuselage_height'] = 5.8
    else:
        fuselage_parameter['fuselage_height'] = max(dict_ac_exchange['fuselage_section_total_height'].values())

    ''' set wing mounting position in z direction '''
    # Estimate rear spar position of center wing.
    wing_reference_point_on_center_line_in_z_key, wing_reference_point_on_center_line_in_z_value = \
        next(iter(dict_ac_exchange['wing_section_position_in_z'].items()), (None, None))
    if wing_reference_point_on_center_line_in_z_value is None:
        dict_ac_exchange['rear_spar_position_center_wing_box_in_z_direction'] = -0.8
    else:
        dict_ac_exchange['rear_spar_position_center_wing_box_in_z_direction'] = \
            wing_reference_point_on_center_line_in_z_value

    # Estimate main landing gear wing mounting position.
    fuselage_section_width_key, fuselage_section_width_value = \
        next(iter(dict_ac_exchange['fuselage_section_width'].items()), (None, None))
    if dict_ac_exchange['dihedral_angle_wing'] is not None \
            and dict_ac_exchange['rear_spar_position_center_wing_box_in_z_direction'] is not None \
            and fuselage_section_width_value is not None and 'main_gear_outer_strut_y_position' \
            in landing_gear_positions:
        main_gear_arrangement['main_landing_gear_wing_mounting_position_in_z_direction'] = \
            round(dict_ac_exchange['rear_spar_position_center_wing_box_in_z_direction']
                  + ((landing_gear_positions['main_gear_outer_strut_y_position']
                      - max(dict_ac_exchange['fuselage_section_width'].values()) / 2)
                     * math.atan(math.radians(dict_ac_exchange['dihedral_angle_wing']))), 4)
    else:
        if dict_ac_exchange['fuselage_type'] == 'single_aisle':
            main_gear_arrangement['main_landing_gear_wing_mounting_position_in_z_direction'] = -0.9

        if dict_ac_exchange['fuselage_type'] == 'wide_body':
            main_gear_arrangement['main_landing_gear_wing_mounting_position_in_z_direction'] = -1.3

    ''' landing gear load estimation '''
    # Check if the rearmost position of the center of gravity is zero.
    #  -> if true: -> Use main gear x-position minus 3 meter for further estimation.
    if dict_ac_exchange['most_aft_center_of_gravity_in_x_direction'] is None:
        lever_arms['distance_between_nose_gear_and_rearmost_center_of_gravity'] = \
            abs((landing_gear_positions['main_gear_x_position'] - 1)
                - landing_gear_positions['nose_gear_x_position'])
        lever_arms['distance_between_main_gear_and_rearmost_center_of_gravity'] = 1

    # Else condition: The rearmost position of the center of gravity is not zero.
    #  -> Use for further estimation.
    else:
        lever_arms['distance_between_nose_gear_and_rearmost_center_of_gravity'] = \
            abs(dict_ac_exchange['most_aft_center_of_gravity_in_x_direction']
                - landing_gear_positions['nose_gear_x_position'])

    # Distance between nose and main landing gear.
    lever_arms['distance_between_nose_and_main_gear'] = \
        abs(landing_gear_positions['main_gear_x_position'] - landing_gear_positions['nose_gear_x_position'])

    # Minimum static nose gear load.
    landing_gear_loads['nose_gear_minimum_static_load'] = \
        ((landing_gear_loads['maximum_ramp_weight']
          * (lever_arms['distance_between_nose_and_main_gear']
             - lever_arms['distance_between_nose_gear_and_rearmost_center_of_gravity']))
         / lever_arms['distance_between_nose_and_main_gear'])

    landing_gear_loads['percentage_of_nose_gear_minimum_load'] = \
        (landing_gear_loads['nose_gear_minimum_static_load'] / landing_gear_loads['maximum_ramp_weight']) * 100

    # Estimation of total main gear load (100 percent - minimum_nose_gear_load_in_percent)/100 * maximum_ramp_weight
    landing_gear_loads['main_gear_maximum_total_load'] = \
        ((100 - landing_gear_loads['percentage_of_nose_gear_minimum_load'])
         / 100 * landing_gear_loads['maximum_ramp_weight'])

    ''' set outputs to landing gear description dictionary '''
    lever_arms = {key: round(value, 4) for key, value in lever_arms.items()}
    landing_gear_loads = {key: round(value, 4) for key, value in landing_gear_loads.items()}
    fuselage_parameter = {key: round(value, 4) for key, value in fuselage_parameter.items()}
    landing_gear_positions = {key: round(value, 4) for key, value in landing_gear_positions.items()}
    landing_gear_description['lever_arms'] = lever_arms
    landing_gear_description['landing_gear_loads'] = landing_gear_loads
    landing_gear_description['fuselage_parameter'] = fuselage_parameter
    landing_gear_description['landing_gear_positions'] = landing_gear_positions
    landing_gear_description['nose_gear_description']['nose_gear_tires'] = nose_gear_tires
    landing_gear_description['nose_gear_description']['nose_gear_arrangement'] = nose_gear_arrangement
    landing_gear_description['main_gear_description']['main_gear_tires'] = main_gear_tires
    landing_gear_description['main_gear_description']['main_gear_arrangement'] = main_gear_arrangement
    landing_gear_description['tail_tipping_point']['maximum_rotation_angle_landing'] = maximum_rotation_angle_landing
    landing_gear_description['landing_gear_limitations']['maximum_tail_tipping_angle'] = maximum_rotation_angle_landing

    return landing_gear_description
