/**
 * \file multi_engine_propulsion.cpp
 * \author Gerrit Pauls (gerrit.pauls@tuhh.de)
 * \brief
 * \version 0.1
 * \date 2024-08-30
 *
 * @copyright Copyright (c) 2024
 *
 */

#include "multi_engine_propulsion.h"
#include <algorithm>

/* Initialization of attributes of the Multi_engine_propulsion object with the constructor */
Multi_engine_propulsion::Multi_engine_propulsion(const std::shared_ptr<RuntimeIO>& rtIO) {
    // Set up needed nodes
    const node& propulsor_requirements = rtIO->acxml.at("requirements_and_specifications/design_specification/propulsion");
    const node& propulsor_design = rtIO->acxml.at("component_design/propulsion/specific");
    const node& tank_definition = rtIO->acxml.at("requirements_and_specifications/design_specification/configuration/tank_definition");
    const node& tank_design = rtIO->acxml.at("component_design/tank/specific");

    for (uint16_t i(0); i < propulsor_design.getVector("propulsion").size(); i++) {
        // get the energy_carrier_id of the propulsor
        int id_tmp = EndnodeReadOnly<int>("/propulsor@" + num2Str(i) + "/energy_carrier_ID").read(propulsor_requirements).value();

        // get all characteristic for its engine
        double scale_tmp(EndnodeReadOnly<double>("/propulsion@" + num2Str(i) + "/engine/scale_factor").read(propulsor_design).value());
        std::string name_tmp = EndnodeReadOnly<std::string>("/propulsion@" + num2Str(i) + "/engine/model").read(propulsor_design).value();
        double thrust_share_tmp(EndnodeReadOnly<double>("/propulsor@" + num2Str(i) + "/thrust_share").read(propulsor_requirements).value());

        // Check if the energy_carrier_id is already used
        if (this->propulsors.find(id_tmp) != this->propulsors.end()) {
            // Check if the same engine already operates using that energy carrier
            for (uint16_t j(0); j < this->propulsors[id_tmp].size(); j++) {
                if (this->propulsors[id_tmp][j].name == name_tmp // check for same name and scale factor
                        && accuracyCheck(this->propulsors[id_tmp][j].engine.get_scale_factor(), scale_tmp, ACCURACY_LOW)) {
                    this->propulsors[id_tmp][j].engine.set_same_engine_type_quantity(1 +
                        this->propulsors[id_tmp][j].engine.get_same_engine_type_quantity()); // add them up
                    this->propulsors[id_tmp][j].thrust_share += thrust_share_tmp; // sum their thrust share
                }
            }
        } else { // if not make a new one
            Propulsor propulsor_tmp(name_tmp, Engine(rtIO->getEngineDataDir() + "/" + name_tmp, scale_tmp), thrust_share_tmp);
            this->propulsors[id_tmp].push_back(propulsor_tmp);
        }
    }

    /* Get all tanks and gather them with corresponding energy carrier IDs */
    // Read tank IDs and capacities from the aircraft XML
    std::map<int, double> total_capacity_per_id;
    for (int tank_id(0); tank_id < tank_design.getVector("tank").size(); tank_id++) {
        int energy_carrier_id(EndnodeReadOnly<int>("energy_carrier_ID").read(tank_definition.at("tank@" + num2Str(tank_id))).value());
        check_energy_carrier_id(energy_carrier_id, "Problematic tank setup. ");
        double tank_capacity(EndnodeReadOnly<double>("maximum_energy_capacity").read(tank_design.at("tank@" + num2Str(tank_id))).value());
        Tank tmp_tank(tank_id, tank_capacity, rtIO->get_fuel_density(energy_carrier_id),
                        rtIO->get_fuel_gravimetric_energy_density(energy_carrier_id),
                        rtIO->get_fuel_volumetric_energy_density(energy_carrier_id));
        this->tanks[energy_carrier_id].push_back(tmp_tank);
        total_capacity_per_id[energy_carrier_id] += tank_capacity; // sum capacity of this energy carrier
    }

    // Get the relative maximum capacity of each tank in comparison to other tanks using the same energy carrier ID
    for (std::pair<int, std::vector<Tank>> tank : this->tanks) {
        for (uint16_t i(0); i < tank.second.size(); i++) {
            tank.second[i].relative_capacity = tank.second[i].capacity/total_capacity_per_id[tank.first];
        }
    }

    // Final check if all propulsors will get energy from at least one tank
    for (std::pair<int, std::vector<Propulsor>> tmp : propulsors) {
        if (this->tanks.find(tmp.first) == tanks.end()) {
            myRuntimeInfo->out << tmp.first << std::endl;
            throwError(__FILE__, __func__, __LINE__,
                std::format("No tank was designed containing the energy carrier {}: No fuel for engine {}!", tmp.first, tmp.second[0].name));
        }
    }
}

/* Destructor of the flight conditions object */
Multi_engine_propulsion::~Multi_engine_propulsion() {
}

size_t Multi_engine_propulsion::unique_engines_per_id(const int& energy_carrier_id) {
    check_energy_carrier_id(energy_carrier_id);
    return this->propulsors[energy_carrier_id].size();
}

std::vector<std::string> Multi_engine_propulsion::get_engine_names(const int& energy_carrier_id) {
    check_energy_carrier_id(energy_carrier_id);
    std::vector<std::string> names;
    std::transform(this->propulsors[energy_carrier_id].begin(), this->propulsors[energy_carrier_id].end(), std::back_inserter(names),
                    [](const auto& engines) { return engines.name; });
    return names;
}

std::vector<Engine> Multi_engine_propulsion::get_engines(const int& energy_carrier_id) {
    check_energy_carrier_id(energy_carrier_id);
    std::vector<Engine> engines_objects;
    std::transform(this->propulsors[energy_carrier_id].begin(), this->propulsors[energy_carrier_id].end(), std::back_inserter(engines_objects),
                    [](const auto& engines) { return engines.engine; });
    return engines_objects;
}

std::vector<double> Multi_engine_propulsion::get_thrust_shares(const int& energy_carrier_id) {
    check_energy_carrier_id(energy_carrier_id);
    std::vector<double> thrust_shares;
    std::transform(this->propulsors[energy_carrier_id].begin(), this->propulsors[energy_carrier_id].end(), std::back_inserter(thrust_shares),
                    [](const auto& engines) { return engines.thrust_share; });
    return thrust_shares;
}

std::vector<OperatingPoint> Multi_engine_propulsion::get_operating_points(const int& energy_carrier_id) {
    check_energy_carrier_id(energy_carrier_id);
    std::vector<OperatingPoint> operating_points;
    for (uint16_t i(0); i < this->propulsors[energy_carrier_id].size(); i++) {
        operating_points.push_back(this->propulsors[energy_carrier_id][i].engine.get_operating_point());
    }
    return operating_points;
}

void Multi_engine_propulsion::set_operating_points(const int& energy_carrier_id, std::vector<OperatingPoint> operating_points) {
    check_energy_carrier_id(energy_carrier_id);
    for (uint16_t i(0); i < this->propulsors[energy_carrier_id].size(); i++) {
        this->propulsors[energy_carrier_id][i].engine.set_operating_point(operating_points[i]);
    }
}

double Multi_engine_propulsion::get_aircraft_fuelflow(const int& energy_carrier_id) {
    check_energy_carrier_id(energy_carrier_id);
    double total_fuel_flow(0);
    // Loop through all propulsors running on that energy carrier
    for (uint16_t i(0); i < this->propulsors[energy_carrier_id].size(); i++) {
        total_fuel_flow += this->propulsors[energy_carrier_id][i].engine.get_aircraft_fuelflow();
    }
    return total_fuel_flow;
}

double Multi_engine_propulsion::get_thrust_aircraft(const int& energy_carrier_id) {
    check_energy_carrier_id(energy_carrier_id);
    double total_thrust(0);
    for (uint16_t i(0); i < this->propulsors[energy_carrier_id].size(); i++) {
        total_thrust += this->propulsors[energy_carrier_id][i].engine.get_thrust_aircraft();
    }
    return total_thrust;
}

void Multi_engine_propulsion::calculate_N1_with_penalties(const int& energy_carrier_id, const double& altitude,
                                             const double& mach_number, const atmosphere& atm,
                                             const double& derate, const std::string& thrust_rating,
                                             double bleed_air_offtake, double shaft_power_offtake) {
    for (uint16_t i(0); i < this->propulsors[energy_carrier_id].size(); i++) {
        this->propulsors[energy_carrier_id][i].engine.calculate_N1_with_penalties(
                                                        altitude, mach_number, atm, derate,
                                                        thrust_rating, bleed_air_offtake, shaft_power_offtake);
    }
}

void Multi_engine_propulsion::calculate_N1_with_thrustlimit(const int& energy_carrier_id, const double& altitude,
                                             const double& mach_number, const atmosphere& atm,
                                             const double& derate, const std::string& thrust_rating,
                                             double bleed_air_offtake, double shaft_power_offtake, double thrust_limit) {
    for (uint16_t i(0); i < this->propulsors[energy_carrier_id].size(); i++) {
        double thrust_limit_share = this->propulsors[energy_carrier_id][i].thrust_share*thrust_limit;
        this->propulsors[energy_carrier_id][i].engine.calculate_N1_with_thrustlimit(
                                                        altitude, mach_number, atm, derate, thrust_rating,
                                                        bleed_air_offtake, shaft_power_offtake, thrust_limit_share);
    }
}

std::vector<Multi_engine_propulsion::Tank> Multi_engine_propulsion::get_tanks_by_id(int energy_carrier_id) {
    check_energy_carrier_id(energy_carrier_id);
    return this->tanks[energy_carrier_id];
}

void Multi_engine_propulsion::drain_tanks(const int& energy_carrier_id, const double& consumed_energy) {
    check_energy_carrier_id(energy_carrier_id);
    for (int i(0); i < this->tanks[energy_carrier_id].size(); i++) {
            this->tanks[energy_carrier_id][i].used_energy -= this->tanks[energy_carrier_id][i].relative_capacity*consumed_energy;
    }
}

double Multi_engine_propulsion::get_consumed_fuel_mass() {
    double total_fuel_mass(0);
    for (std::pair<const int, std::vector<Multi_engine_propulsion::Tank>> id : this->tanks) {
        total_fuel_mass += // id.first = energy_carrier_id
            std::accumulate(this->tanks[id.first].begin(), this->tanks[id.first].end(), 0.0,
                            [](double sum, const Tank& tank) {return sum + (tank.used_energy / tank.gravimetric_density);});
    }
    return total_fuel_mass;
}

double Multi_engine_propulsion::get_total_energy_capacity(const int& energy_carrier_id) {
    check_energy_carrier_id(energy_carrier_id);
    double total_energy(0);
    for (std::pair<const int, std::vector<Multi_engine_propulsion::Tank>> id : this->tanks) {
        total_energy += // id.first = energy_carrier_id
            std::accumulate(this->tanks[id.first].begin(), this->tanks[id.first].end(), 0.0,
                            [](double sum, const Tank& tank) {return sum + (tank.capacity);});
    }
    return total_energy;
}

/* Initialization of attributes of the Propulsor object with the constructor */
Multi_engine_propulsion::Propulsor::Propulsor(const std::string& a_name, Engine an_engine, const double& a_thrust_share) :
    name(a_name),
    engine(an_engine),
    thrust_share(a_thrust_share) {
}

/* Destructor of the Propulsor object */
Multi_engine_propulsion::Propulsor::~Propulsor() {
}

/* Initialization of attributes of the Tank object with the constructor */
Multi_engine_propulsion::Tank::Tank(uint16_t a_tank_id, double a_capacity, double a_density, double a_gravimetric_density, double a_volumetric_density) :
    tank_id(a_tank_id),
    capacity(a_capacity),
    relative_capacity(0.),
    used_energy(0.),
    density(a_density),
    gravimetric_density(a_gravimetric_density),
    volumetric_density(a_volumetric_density) {
}

/* Destructor of the Tank object */
Multi_engine_propulsion::Tank::~Tank() {
}

void Multi_engine_propulsion::check_energy_carrier_id(const int& energy_carrier_id, const std::string additional_info) {
    if (this->propulsors.find(energy_carrier_id) == propulsors.end()) {
        throwError(__FILE__, __func__, __LINE__,
            std::format("{}Check yor energy carriers: Given energy carrier ID is not used for this propulsion system!", additional_info));
    }
}
