/**
 * \file multi_engine_propulsion.h
 * \author Gerrit Pauls (gerrit.pauls@tuhh.de)
 * \brief
 * \version 0.1
 * \date 2024-08-30
 *
 * @copyright Copyright (c) 2024
 *
 */

#ifndef performanceAssessment_SRC_LIBS_MULTI_ENGINE_PROPULSION_H_
#define performanceAssessment_SRC_LIBS_MULTI_ENGINE_PROPULSION_H_

#include <map>
#include <memory>
#include <string>
#include <tuple>
#include <utility>
#include <vector>

#include <aixml/endnode.h>
#include <engine/engine.h>
#include <moduleBasics/module.h>

/** \brief Class for flight conditions
 */
class Multi_engine_propulsion {
 public:
    /* Constructor and destructor */
    /** \brief Constructor for class Multi_engine_propulsion
     */
    explicit Multi_engine_propulsion(const std::shared_ptr<RuntimeIO>& rtIO);
    /** \brief Destructor for class Multi_engine_propulsion
     */
    virtual ~Multi_engine_propulsion();

    size_t unique_engines_per_id(const int& energy_carrier_id);
    std::vector<std::string> get_engine_names(const int& energy_carrier_id);
    std::vector<Engine> get_engines(const int& energy_carrier_id);
    std::vector<double> get_thrust_shares(const int& energy_carrier_id);

    std::vector<OperatingPoint> get_operating_points(const int& energy_carrier_id);
    void set_operating_points(const int& energy_carrier_id, std::vector<OperatingPoint> operating_points);

    double get_aircraft_fuelflow(const int& energy_carrier_id);
    double get_thrust_aircraft(const int& energy_carrier_id);

    void calculate_N1_with_penalties(const int& energy_carrier_id,
                                     const double& altitude,
                                     const double& mach_number,
                                     const atmosphere& atm,
                                     const double& derate,
                                     const std::string& thrust_rating,
                                     double bleed_air_offtake,
                                     double shaft_power_offtake);
    void calculate_N1_with_thrustlimit(const int& energy_carrier_id,
                                       const double& altitude,
                                       const double& mach_number,
                                       const atmosphere& atm,
                                       const double& derate,
                                       const std::string& thrust_rating,
                                       double bleed_air_offtake,
                                       double shaft_power_offtake,
                                       double thrust_limit);

    class Propulsor {
     public:
        std::string name;
        Engine engine;
        double thrust_share;

        /** \brief Constructor of used_components.
         */
        Propulsor(const std::string& a_name, Engine an_engine, const double& a_thrust_share);

        /** \brief Destructor of used_components
         */
        virtual ~Propulsor();
    };

    class Tank {
     public:
        uint16_t tank_id;
        double capacity;
        double relative_capacity;
        double used_energy;
        double density;
        double gravimetric_density;
        double volumetric_density;

        /** \brief Constructor of used_components.
         */
        Tank(uint16_t a_tank_id, double a_capacity, double a_density, double a_gravimetric_density, double a_volumetric_density);

        /** \brief Destructor of used_components
         */
        virtual ~Tank();
    };
    std::vector<Tank> get_tanks_by_id(int energy_carrier_id);
    void tank_setup(const node& tank_definition, const node& tank_design);
    void drain_tanks(const int& energy_carrier_id, const double& consumed_energy);
    double get_consumed_fuel_mass();
    double get_total_energy_capacity(const int& energy_carrier_id);

 private:
    /* variables */
    std::map<int, std::vector<Propulsor>> propulsors;
    std::map<int, std::vector<Tank>> tanks;

    /* functions */
    void check_energy_carrier_id(const int& energy_carrier_id, const std::string additional_info = "");
};

#endif  // performanceAssessment_SRC_LIBS_MULTI_ENGINE_PROPULSION_H_
