/*  Copyright (C) 2012-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "fuelFromMassFraction.h"

using std::endl;
using std::vector;

fuelFromMassFraction::fuelFromMassFraction(double thisOME, double cruiseMach)
    :
    theBreguetFactor(0.),
    breguetEfficiencyBlockFuel(1.0),
    breguetEfficiencyTripFuel(1.0),
    missionFuel(0.),
    tripFuel(0.),
    theOME(thisOME),
    theCruiseMach(cruiseMach) {
    // ctor
}

fuelFromMassFraction::~fuelFromMassFraction() {
    // dtor
}

// TODO(franz#1#): evtl ueberfluessig, die mit calcFuelFromMassFractions wird einfach so angepasst mit den breguetEfficiency-Faktoren,
// damit mission-/tripfuel gleich dem Fuel von calcStandardFuelPlanning in fuelPlanning

void fuelFromMassFraction::calibrateBreguet(double targetMissionFuel, double targetTripFuel, double designPayload, double designRange) {
    calcFuelFromMassFractions(designPayload, designRange);
    breguetEfficiencyBlockFuel = targetMissionFuel / missionFuel;
    breguetEfficiencyTripFuel = targetTripFuel / tripFuel;
    calcFuelFromMassFractions(designPayload, designRange);
    double theDeviation = sqrt(pow((targetMissionFuel - missionFuel) / targetMissionFuel, 2)
                               + pow((targetTripFuel - tripFuel) / targetTripFuel, 2));
    if (theDeviation > 0.01) {
        myRuntimeInfo->err << "Calibration of Breguet fuel estimation not possible!" << endl;
        myRuntimeInfo->err << "Abort!!!" << endl;
        exit(1);
    }
}

double fuelFromMassFraction::iterativeBreguetForRange(double payloadForRange, double takeOffMass, double theMTOM, double theDesignRange) {
    vector<double> ranges;
    vector<double> deltaTOM;
    double TOM(0.);
    ranges.push_back(theDesignRange);
    deltaTOM.push_back(takeOffMass - theMTOM);
    ranges.push_back(0.8 * theDesignRange);
    calcFuelFromMassFractions(payloadForRange, ranges.back());
    TOM = theOME + payloadForRange + tripFuel;
    deltaTOM.push_back(takeOffMass - TOM);
    int i = 1;
    do {
        ranges.push_back(ranges.at(i) - (ranges.at(i) - ranges.at(i - 1)) / (deltaTOM.at(i) - deltaTOM.at(i - 1))*deltaTOM.at(i));
        calcFuelFromMassFractions(payloadForRange, ranges.back());
        TOM = theOME + payloadForRange + tripFuel;
        deltaTOM.push_back(takeOffMass - TOM);
        i++;
        if (deltaTOM.size() > 250) {
            myRuntimeInfo->err << "Error in convergence of iterative Breguet-range calculation!" << endl;
            myRuntimeInfo->err << "Abort!!!" << endl;
            exit(1);
        }
    } while (fabs(deltaTOM.back()) > 0.005);
    double resultRange = ranges.back();
    ranges.clear();
    deltaTOM.clear();
    return resultRange;
}

void fuelFromMassFraction::calcFuelFromMassFractions(double missionPayload, double missionRange) {
    vector<double> segmentMass;
    vector<double> segmentFuel;
    double oldTripFuel(0.);
    tripFuel = 0.;
    missionFuel = 0.;
    do {
        segmentFuel.clear();
        segmentMass.clear();
        double theTOM = theOME + missionPayload + missionFuel;
        oldTripFuel = tripFuel;
        tripFuel = 0.;
        missionFuel = 0.;
        segmentMass.push_back(theTOM);
        for (int i = 1; i <= 9; i++) {
            double fraction(0.);
            /* Take-off */
            if (i == 1) {
                fraction = 0.98;
            } else if (i == 2) { /* Climb and Accelerate */
                fraction = 1.0065 - 0.0325 * theCruiseMach;
            } else if (i == 3) { /* Cruise */
                double thisRange = (missionRange - 300.) * 1000.;  // m
                fraction = pow(2.71828, (-1.) * thisRange * theBreguetFactor);
            } else if (i == 4) { /* Descent */
                fraction = 0.992;
            } else if (i == 5) { /* Aborted Landing -> T-O */
                fraction = 0.98;
            } else if (i == 6) { /* Climb and Accelerate -> Diversion */
                fraction = 1.0065 - 0.0325 * 0.74;
            } else if (i == 7) { /* Cruise Diversion */
                double thisRange = (200. - 100.) * 1000.;  // m
                fraction = pow(2.71828, (-1.) * thisRange * theBreguetFactor);
            } else if (i == 8) { /* Approach Diversion */
                fraction = 0.992;
            } else if (i == 9) { /* Landing */
                fraction = 0.995;
            }
            segmentFuel.push_back((1. - fraction) * segmentMass.at(i - 1));
            segmentMass.push_back(segmentMass.at(i - 1) - segmentFuel.back());
            tripFuel += segmentFuel.back();
            if (i <= 5) {
                missionFuel += segmentFuel.back();
            }
            if (segmentMass.back() < 0.) {
                exit(1);
            }
        }
    } while (fabs(oldTripFuel - tripFuel) / tripFuel >= 0.0001);
    segmentFuel.clear();
    segmentMass.clear();
    missionFuel = breguetEfficiencyBlockFuel * missionFuel;
    tripFuel = breguetEfficiencyTripFuel * tripFuel;
}
