/*  Copyright (C) 2012-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef performance_assessment_SRC_TAW_DEFAULTMETHODS_FUELPLANNING_LOW_FIDELITY_FUELPLANNING_H_
#define performance_assessment_SRC_TAW_DEFAULTMETHODS_FUELPLANNING_LOW_FIDELITY_FUELPLANNING_H_

#include <moduleBasics/module.h>
#include <unitConversion/unitConversion.h>
#include <standardFiles/functions.h>

#include <memory>
#include <vector>
#include <string>
#ifdef _WIN32
    #include <windows.h>
#endif  // _WIN32
#include <numeric>

#include "../abstractFuelPlanning.h"
#include "../../SARPerformance/abstractSARPerformance.h"

#include "../../performance_assessment_settings.h"
#include "../../aircraft.h"
#include "flightSimulator.h"
#include "fuelFromMassFraction.h"

#include "../../SARPerformance/low_fidelity/SARPerformance.h"  // <-- questionable

namespace taw {
class performance_assessment_factory;
}

namespace low {

class fuelPlanning : public ::abstractFuelPlanning {
 public:

    /**\brief Method for the fuel planning
    *
    */
    void doFuelPlanning() override;

    /**\brief Constructor fuelPlanning
    *\param theAircraft Address to the aircraft object
    *\param rtIO pointer to the runtimeIO object
    */
    fuelPlanning(aircraft *theAircraft, const performance_assessment_settings *theSettings, const std::shared_ptr<RuntimeIO>& rtIO);

    /**\brief Destructor fuelPlanning
    *
    */
    virtual ~fuelPlanning() { }

    // /**\class payloadRangeData Class with parameters for the payload range diagram
    // *
    // */
    // class payloadRangeData {
    //  public:
    //     double Range; /**< Distance covered in the mission based on the payload-range diagram (NM) */
    //     double Payload; /**< Payload carried by the aircraft (kg) */
    //     double Fuel; /**< Fuel carried by the aircraft (kg) */
    //     double TOM; /**< Aircraft take-off mass (kg) */
    //     payloadRangeData() {
    //         Range = 0.0;
    //         Payload = 0.0;
    //         Fuel = 0.0;
    //         TOM = 0.0;
    //     }
    // };
    // vector<payloadRangeData> myPayloadRange;  /**< List of values of the payload-range diagram */

    // double tripTime;  /**< Trip time (min) */
    // double blockTime; /**< Block time (min) */

    /**\brief Calculates the data for the payload range diagram
    *
    */
    void doPayloadRangeOLD() override;  // PayloadRange-Calculation if FuelPlanning on; otherwise see payloadRange class

    /**\brief Creates plot for maximum payload
    *
    */
    void doFuelPayloadRangeOLD() override;  // writes fuelPayloadRange.csv for Plot

    /**\brief Reads the payload-range data from the AcftXML
    *
    */
    void payloadRangeFromAcftXML() override;

 private:
    const std::shared_ptr<RuntimeIO>& rtIO;
    std::shared_ptr<aircraft> theAcftPt; /**< Pointer to aircraft object */
    const performance_assessment_settings *theSettingsPt; /**< Pointer to the performance_assessment_settings object */
    flightSimulator myFlightSim; /**< flightSimulator object */
    fuelFromMassFraction myBreguet; /**< fuelFromMassFraction object */
    double relConFuel; /**< Relative amount of contingency fuel in reference to the total trip fuel (-) */

    /**\class pointData Class containing data for the various mission points
    *
    */
    class pointData {
     public:
        double Mach; /**< Mach number (-) */
        double Altitude; /**< Aircraft altitude (m) */
        double TAS; /**< True air speed (m/s) */
        double ROC; /**< Aircraft rate of climb (m/s) */
        double derate; /**< Artificial constant to reduce the engine spool speed (between 0 and 1) */
        double bleedOff; /**< Current bleed air offtake (kg/s) */
        double shaftOff; /**< Current shaft power offtake (kW) */

        string config; /**< Flap configuration of the aircraft */
        string rating; /**< Engine rating (Take-Off, MaxCont, Climb, Idle, Cruise) */

        pointData()
            :
            config(""),
            rating("") {
                Mach = 0.0;
                Altitude = 0.0;
                TAS = 0.0;
                ROC = 0.0;
                derate = 0.0;
                bleedOff = 0.0;
                shaftOff = 0.0;
        }
    };
    pointData P;

    /**\brief Method to calculate the standard fuel planned for the flight
    *\param theTaxiFuelTakeoff Taxiing fuel at takeoff (kg)
    *\param theTaxiFuelLanding Taxiing fuel at landing (kg)
    *\param theMissionFuel Mission fuel (kg)
    *\param theContigencyFuel Contigency fuel (kg)
    *\param theAlternateFuel Alternate fuel (kg)
    *\param theFinalReserveFuel Final reserve fuel (kg)
    *\param theTripFuel Trip fuel (kg)
    *\param theRange Aircraft range (km)
    *\param thePayload Aircraft payload (kg)
    *\param writingMissionData Switch if the fuel mission data shall be written
    *\param myAERO Object of the aerodynamics class
    */
    void calcStandardFuelPlanning(double *theTaxiFuelTakeoff, double *theTaxiFuelLanding, double *theMissionFuel, double *theContigencyFuel,
                                  double *theAlternateFuel, double *theFinalReserveFuel, double *theTripFuel,
                                  double theRange, double thePayload, bool writingMissionData,
                                  const aerodynamics &myAERO);

    /**\brief Function to calculate the block fuel
    *\param acMass Aircraft mass (kg)
    *\param Range Aircraft range (NM)
    *\param writeMission Switch if the fuel mission data shall be written)
    *\param theAERO Object of the aerodynamics class
    *\return Block fuel (kg)
    */
    double calcMissionFuel(double acMass, double Range, bool writeMission, aerodynamics theAERO);

    /**\brief Function to calculate the alternate fuel
    *\param acMass Aircraft mass (kg)
    *\param theAERO Object of the aerodynamics class
    *\return Alternate fuel (kg)
    */
    double calcAlternateFuel(double acMass, aerodynamics theAERO);

    /**\brief Function to calculate the final reserve fuel
    *\param acMass Aircraft mass (kg)
    *\param theAERO Object of the aerodynamics class
    *\return Final reserve fuel (kg)
    */
    double calcFinalReserveFuel(double acMass, aerodynamics theAERO);

    /**\brief Method to calculate the trip fuel
    *\param WO Take-Off mass (kg)
    *\param Time Cruise duration at 10000 ft (s)
    *\param theSFC Specific fuel consumption (kg/kN*s)
    *\param theLoD glide ratio
    *\return Fuel mass (kg)
    */
    double calcBreguet(double W0, double Time, double theSFC, double theLoD);

    /**\brief Function to calculate the speed for the highest specific air range (SAR)
    *\param alt Aircraft altitude (m)
    *\param mass Aircraft mass (kg)
    *\param lowerSpeed Lowest aircraft speed (m/s)
    *\param upperSpeed Highest aircraft speed (m/s)
    *\return Mach number (-)
    */
    double calcBestSARSpeed(double alt, double mass, double lowerSpeed, double upperSpeed);

//    /**\brief Function calculates the remaining range to reach a target mass
//    *\param Range0 First range value (NM)
//    *\param Range1 Second range value (NM)
//    *\param Mass0 First mass value (kg)
//    *\param Mass1 Second mass value (kg)
//    *\param targetMass Target mass (kg)
//    *\return Remaining range (NM)
//    */
//    double calcRangeForMass(double Range0, double Range1, double Mass0, double Mass1, double targetMass);

    /**\brief Function sets a mission point
    *\param ID Mission segment
    */
    void setP(int ID);

//    /**\brief Function checks if the thrust allows the required ROC
//    *\param ID Mission segment
//    *\param theP Object of the pointData class
//    *\param massAircraft Aircraft mass (kg)
//    *\param theAERO Object of the aerodynamics class
//    */
//    void exitROC(int ID, pointData theP, double massAircraft, aerodynamics theAERO);

    /**\brief Function checks if the thrust allows the required acceleration
    *\param ID Mission segment
    *\param theP Object of the pointData class
    *\param massAircraft Aircraft mass (kg)
    */
    void exitAcceleration(int ID, pointData theP, double massAircraft);

    /**\brief Function checks if the thrust allows a steady level flight
    *\param ID Mission segment
    *\param theP Object of the pointData class
    *\param theDrag Drag (kN)
    */
    void exitSteadyLevelFlight(int ID, pointData theP, double theDrag);

    /**\brief Function estimates the quantity of fuel
    *\param theAERO Object of the aerodynamics class
    *\return Fuel mass (kg)
    */
    double fastFuelEstimate(aerodynamics theAERO);

};

}  // namespace low
#endif  // performance_assessment_SRC_TAW_DEFAULTMETHODS_FUELPLANNING_LOW_FIDELITY_FUELPLANNING_H_
