/*  Copyright (C) 2012-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "performance_assessment_settings.h"

using std::endl;

performance_assessment_settings::performance_assessment_settings(const std::shared_ptr<RuntimeIO>& rtIOin)
    :
    rtIO(rtIOin),
    designForMTOM(rtIO->moduleConfig.at("/module_configuration_file/program_settings/modes/mtom_design")),
    useStudyMissionForAnalysis(rtIO->moduleConfig.at("/module_configuration_file/program_settings/modes/use_study_mission_for_analysis")),  // CSV-Output?
    optimizeMission(rtIO->moduleConfig.at("/module_configuration_file/program_settings/mission/optimize_mission_profile")) {

    /* Constants for start/runway calculations */
    myField.runwaySlope = PI / 180. * EndnodeReadOnly<double>("/module_configuration_file/program_settings/constants_for_performance_checks/field_performance/runway_slope")
        .read(rtIO->moduleConfig).value();
    myField.headWind = EndnodeReadOnly<double>("/module_configuration_file/program_settings/constants_for_performance_checks/field_performance/head_wind")
        .read(rtIO->moduleConfig).value();
    /* Constants for the Representation of the Vn Diagram */
    myVnDiagramConstants.maxNClean = EndnodeReadOnly<double>("/module_configuration_file/program_settings/constants_for_performance_checks/vn_diagram/max_n_clean")
        .read(rtIO->moduleConfig).value();
    myVnDiagramConstants.minNClean = EndnodeReadOnly<double>("/module_configuration_file/program_settings/constants_for_performance_checks/vn_diagram/min_n_clean")
        .read(rtIO->moduleConfig).value();
    myVnDiagramConstants.maxNFlaps = EndnodeReadOnly<double>("/module_configuration_file/program_settings/constants_for_performance_checks/vn_diagram/max_n_flaps")
        .read(rtIO->moduleConfig).value();
    myVnDiagramConstants.minNFlaps = EndnodeReadOnly<double>("/module_configuration_file/program_settings/constants_for_performance_checks/vn_diagram/min_n_flaps")
        .read(rtIO->moduleConfig).value();
    /* Engine Performance Settings */
    int nPerfSettings = rtIO->moduleConfig.getVector("/module_configuration_file/program_settings/constants_for_performance_checks/engine_performance/ratings/rating", 1).size();
    auto aVector = rtIO->moduleConfig.getVector("/module_configuration_file/program_settings/constants_for_performance_checks/engine_performance/ratings", 1);
    for (int i = 0; i <= nPerfSettings - 1; i++) {
        string tempRating = rtIO->moduleConfig.at("/module_configuration_file/program_settings/constants_for_performance_checks/engine_performance/ratings/rating@"
            + num2Str(i+1)+"/value");
        enginePerformance.Rating.push_back(tempRating);
    }
    /* Switch for Performance Checks */
    mySwitches.doPayloadRange = rtIO->moduleConfig.at("/module_configuration_file/program_settings/performance_checks/payload_range_diagram/switch");
    mySwitches.doEnginePerformance = rtIO->moduleConfig.at("/module_configuration_file/program_settings/performance_checks/engine_performance/switch");
    mySwitches.doFlightEnvelopePerformance = rtIO->moduleConfig.at("/module_configuration_file/program_settings/performance_checks/flight_envelope_performance/switch");
    mySwitches.overwriteInitialValues = rtIO->moduleConfig.at("/module_configuration_file/program_settings/performance_checks/flight_envelope_performance/overwrite_initial_values");
    mySwitches.doClimbPerformance = rtIO->moduleConfig.at("/module_configuration_file/program_settings/performance_checks/climb_performance/switch");
    mySwitches.doTOPerformance = rtIO->moduleConfig.at("/module_configuration_file/program_settings/performance_checks/to_performance/switch");
    if (mySwitches.doTOPerformance) {
        mySwitches.doBLFLPerformance = rtIO->moduleConfig.at("/module_configuration_file/program_settings/performance_checks/to_performance/calculate_blfl_performance");
    } else {
        mySwitches.doBLFLPerformance       = false;
    }
    mySwitches.doLandingPerformance = rtIO->moduleConfig.at("/module_configuration_file/program_settings/performance_checks/landing_performance/switch");
    mySwitches.doVnDiagram = rtIO->moduleConfig.at("/module_configuration_file/program_settings/performance_checks/vn_diagram/switch");
    mySwitches.doFuelPlanning = rtIO->moduleConfig.at("/module_configuration_file/program_settings/fuel_planning/switch");
    /* Fuel Planning */
    myFuelPlan.contingencyFuel = EndnodeReadOnly<double>("/module_configuration_file/program_settings/fuel_planning/contingency_fuel").read(rtIO->moduleConfig).value();
    
    /* Area 51 */
    node& polarXML = aixml::openDocument(rtIO->getIODir() + "/aeroData/" + rtIO->acxml.at("/aircraft_exchange_file/analysis/aerodynamics/polar/polar_file/value"));
    std::string missionxmlPath = rtIO->acxml.at("/aircraft_exchange_file/requirements_and_specifications/mission_files/design_mission_file/value");
    std::string missionString = rtIO->getIODir() + "/missionData/" + rtIO->acxml.at(
        "/aircraft_exchange_file/requirements_and_specifications/mission_files/design_mission_file/value");
    node& missionXMLnode = aixml::openDocument(missionString);
    // std::string aaa = static_cast<std::string>(rtIO->acxml.at("/aircraft_exchange_file/requirements_and_specifications/mission_files/study_mission_file/value"));
    // node& missionXMLtest = aixml::openDocument(aaa);
}

void performance_assessment_settings::checkForValidity(double Value, double lowerBound, double upperBound, const string &Text) {
    if (std::isnan(Value)) {  // Test for NaN
        myRuntimeInfo->err << "Value is NaN!" << endl;
        myRuntimeInfo->err << "Parameter off:" << endl;
        myRuntimeInfo->err << Text << endl;
        myRuntimeInfo->err << "Abort!" << endl;
        exit(1);
    }
    if (std::isinf(Value)) {  // Test for real number
        myRuntimeInfo->err << "Value is Infinite!" << endl;
        myRuntimeInfo->err << "Parameter aus:" << endl;
        myRuntimeInfo->err << Text << endl;
        myRuntimeInfo->err << "Abort!" << endl;
        exit(1);
    }
    if (Value > upperBound || Value < lowerBound) {
        myRuntimeInfo->err << "Invalid value range!" << endl;
        myRuntimeInfo->err << "Parameter off:" << endl;
        myRuntimeInfo->err << Text << endl;
        myRuntimeInfo->err << "With the value:" << endl;
        myRuntimeInfo->err << Value << " (permitted is " << lowerBound << " <= X <= " << upperBound << ")" << endl;
        myRuntimeInfo->err << "Abort!" << endl;
        exit(1);
    }
}

performance_assessment_settings::~performance_assessment_settings() {
}