/*  Copyright (C) 2023 Chair of Aircraft Design, Technical University Munich
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * @file propulsion_strategy.h
 * @author Sebastian Oberschwendtner (sebastian.oberschwendtner@tum.de)
 * @brief The strategy specialization for the propulsionDesign tool.
 * @version 3.0.0
 * @date 2023-12-14
 */

#ifndef SRC_PROPULSION_STRATEGY_H_
#define SRC_PROPULSION_STRATEGY_H_

/* === Includes === */
#include <memory>
#include <moduleBasics/strategySelector.h>
#include <aixml/node.h>
#include "propulsion.h"

/* === Classes === */
/**
 * @class PropulsionStrategy
 * @brief The specialization of the strategy class for the propulsionDesign tool.
 */
class PropulsionStrategy : public Strategy
{
  public:
    /* === Constructors === */
    /**
     * @brief Construct a new PropulsionStrategy instance.
     * @param settings The module settings provided by the user.
     */
    explicit PropulsionStrategy(const std::shared_ptr<node> &configuration) : configuration_(configuration) {}

    /**
     * @brief Destroy the PropulsionStrategy instance.
     */
    ~PropulsionStrategy() override = default;

    /* === Populate the strategy template methods === */
    void initialize() override{};
    void run() override{};
    void update() override{};
    void report() override{};
    void save() override{};

    /* === Define the strategy specialization methods === */
    virtual void operator()(Turbofan<EnergyCarrier::Kerosene> &engine) // NOLINT runtime/reference
    {
        throw std::invalid_argument("The strategy does not yet implement a kerosene turbofan engine.");
    };
    virtual void operator()(Turbofan<EnergyCarrier::Liquid_Hydrogen> &engine) // NOLINT runtime/reference
    {
        throw std::invalid_argument("The strategy does not yet implement a liquid hydrogen turbofan engine.");
    };
    virtual void operator()(Turboprop<EnergyCarrier::Kerosene> &engine) // NOLINT runtime/reference
    {
        throw std::invalid_argument("The strategy does not yet implement a kerosene turboprop engine.");
    };
    virtual void operator()(Turboprop<EnergyCarrier::Liquid_Hydrogen> &engine) // NOLINT runtime/reference
    {
        throw std::invalid_argument("The strategy does not yet implement a liquid hydrogen turboprop engine.");
    };

    /* === Methods === */
    /**
     * @brief Get the configuration of the module.
     *
     * @return std::shared_ptr<node> The configuration of the module.
     */
    [[nodiscard]] auto configuration() const -> const std::shared_ptr<node> { return configuration_; }

  private:
    /* === Properties ===*/
    std::shared_ptr<node> configuration_; /** The configuration of the module. */
};

#endif // SRC_PROPULSION_STRATEGY_H_
