/*  Copyright (C) 2011-2022 Institute of Aerospace Systems, RWTH Aachen University - All rights reserved.
    This file is part of UNICADO.

    UNICADO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UNICADO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UNICADO.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef SYSTEMSDESIGN_SRC_AIRCRAFTSYSTEMS_CONVENTIONALATA25_H_
#define SYSTEMSDESIGN_SRC_AIRCRAFTSYSTEMS_CONVENTIONALATA25_H_

/* The following statements import necessary libraries */
#include <string>                                                   // Import c++ standard library for using string
#include <vector>                                                   // Import c++ standard library for using vector
#include <memory>

/* The following statements import necessary header files */
#include <svl/SVL.h>                                                // Import of header file from src folder for SVL
#include "../aircraftSystem.h"                                      // Import of header file from src folder for aircraftSystem
#include "../powerProfile.h"                                        // Import of header file from src folder for powerProfile
#include "../systemsIOData.h"                                       // Import of header file from systemsIOData for accessing data_. Can't be included in
                                                                    // aircraftSystem.h because that would lead to circular dependencies
#include <aircraftGeometry2/processing/measure.h>

/** \class conventionalATA25
 * \brief class that contains all properties of the furnishings system (ATA25)
 */
class conventionalATA25: public aircraftSystem {
 public:
    int numberOfPowerSources; /**< [-] number of power sources for the furnishing system */
    std::vector<PowerSource> powerSources; /**< Vector containing the power sources of the furnishing system */
    double galleyLoadFractionTakeOff; /**< [-] Workload relative to design Power consumption during Climb [W] **/
    double galleyLoadFractionCruise; /**< [-] Workload relative to design Power consumption during Cruise [W] **/
    double galleyLoadFractionDescent; /**< [-] Workload relative to design Power consumption during Descent [W] **/
    double nonPersonelIFEPower; /**< [W] Basic power consumption of non personal IFE system per PAX (calibrated after [Bue14] p. 59) **/
    double powerPersonelIFE;    /**< [W] Power consumption of the personal IFE system per PAX **/
    double personelIFELoadFractionClimb; /**< [W] Workload relative to design Power consumption during Climb **/
    double personelIFELoadFractionCruise; /**< [W] Workload relative to design Power consumption during Cruise **/
    double personelIFELoadFractionDescent; /**< [W] Workload relative to design Power consumption during Descent **/

    /**
     * \brief function for reading the system specific parameters from the config
     * 
     * \param config const node&: systemsDesign configuration file
    */
    void readParameters(const node& config) override;
    /**
     * \brief function for updating the system specific values in the aircraft xml
    */
    void updateXML() override;

    void calculateSystemsPower(const powerProfile& allPreviousSystemsBleed,
                               const std::vector<powerProfile>& allPreviousSystemsHydraulic,
                               const std::vector<powerProfile>& allPreviousSystemsElectric,
                               const powerProfile& allPreviousSystemsHeat) override;
    void calculateSinkMass(const powerProfile& sourceBleed,
                           const std::vector<powerProfile>& sourceElectric,
                           const std::vector<powerProfile>& sourceHydraulic) override;
    void getPower();

    double getMass();
    Vec3 getRefPoint();

    conventionalATA25(const std::string& systemName, bool systemIsOperating, std::shared_ptr<systemsIOData> data, const RuntimeIO& rtIO);
    virtual ~conventionalATA25();
};

#endif // SYSTEMSDESIGN_SRC_AIRCRAFTSYSTEMS_CONVENTIONALATA25_H_
