"""Module providing calculation functions provided by the user."""
# Import standard modules.
import sys

# Import own modules.
from src.tube_and_wing.empirical.tank_design_tu_berlin.general.tankdesigntuberlin import tank_design_tu_berlin
from src.tube_and_wing.empirical.tank_design_tu_berlin.liquid_hydrogen.calculateconicaltank import calculate_conical_tank
from src.tube_and_wing.empirical.tank_design_tu_berlin.liquid_hydrogen.calculatecylindricaltank import calculate_cylindrical_tank
from src.tube_and_wing.empirical.tank_design_tu_berlin.general.calculatecenterofgravity import calculate_center_of_gravity


def method_liquid_hydrogen(paths_and_names, routing_dict, dict_ac_exchange, dict_mod_config, runtime_output):
    """Tank design method for liquid hydrogen (LH2)-powered tube-and-wing (TAW) aircraft.

    This function performs the tank design for liquid hydrogen-powered TAW aircraft configurations.
    [Add more information here...]
    The output dictionary 'liquid_hydrogen_output_dict' contains the results of the tank design and is structured
    according to the following scheme:
        liquid_hydrogen_output_dict = {'group_name_1': {'parameter_1': value, 'parameter_2': value, ...},
                                       'group_name_2': {'parameter_1': value, 'parameter_2': value, ...},
                                       ...}

    :param dict paths_and_names: Dictionary containing system paths and ElementTrees
    :param dict routing_dict: Dictionary containing routing parameters
    :param dict dict_ac_exchange: Dict containing parameters and according values from aircraft exchange file
    :param dict dict_mod_config: Dict containing parameters and according values from module configuration file
    :param logging.Logger runtime_output: Logging object used for capturing log messages in the module
    :return dict liquid_hydrogen_output_dict: Dictionary containing results from calculation for LH2-powered aircraft
    """

    # Merge both data dictionaries.
    dict_complete_data = dict_ac_exchange | dict_mod_config
    # Extract general parameters from 'complete_data_dict'.
    dict_general_data = {key: value for key, value in dict_complete_data.items() if
                         not key.endswith('_design') and not key.endswith('_study')}
    # Extract design mission related parameters from 'complete_data_dict' and save in 'design_data' (w/o ending).
    dict_design_data = {key.replace('_design', ''): value for key, value in dict_complete_data.items()
                        if key.endswith('_design')}
    # Iterate over dictionary.
    for key, value in dict_design_data.items():
        # Check, if any value is 'None'.
        if value is None:
            # If any value is 'None', the tank design is impossible and the program aborted.
            runtime_output.critical('Error: Tank design not possible due to missing parameters! '
                                    + 'Program aborted.')
            sys.exit(1)
    # Merge design and general data dictionary.
    dict_design = dict_general_data | dict_design_data

    #TODO: Delete this section if data extracted from aircraft exchange file.
    dict_design['geometrical_data'] = {
        # Lengths of aircraft sections.
        'length_nose_section_01': 0.06445732892,
        'length_nose_section_02': 0.1933719868,
        'length_nose_section_03': 0.3222866446,
        'length_nose_section_04': 0.4512013025,
        'length_nose_section_05': 0.5801159603,
        'length_nose_section_06': 0.7090306182,
        'length_nose_section_07': 0.837945276,
        'length_nose_section_08': 0.9668599339,
        'length_nose_section_09': 1.095774592,
        'length_constant_section_01': 32.363204,
        'length_tail_section_01': 1.72542446,
        'length_tail_section_02': 1.72542446,
        'length_tail_section_03': 1.72542446,
        'length_tail_section_04': 1.72542446,
        'length_tail_section_05': 1.72542446,
        'length_tail_section_06': 3.459300377,
        'length_tail_section_07': 3.459300377,
        # Widths of aircraft sections.
        'width_nose_section_01': 0.6219072875,
        'width_nose_section_02': 1.232171749,
        'width_nose_section_03': 1.81877794,
        'width_nose_section_04': 2.368913086,
        'width_nose_section_05': 2.868421677,
        'width_nose_section_06': 3.301028172,
        'width_nose_section_07': 3.6471193,
        'width_nose_section_08': 3.881644273,
        'width_nose_section_09': 3.970066,
        'width_constant_section_01': 3.970066,
        'width_tail_section_01': 3.970066,
        'width_tail_section_02': 3.970066,
        'width_tail_section_03': 3.892703799,
        'width_tail_section_04': 3.618301624,
        'width_tail_section_05': 3.142809701,
        'width_tail_section_06': 1.985033,
        'width_tail_section_07': 0.827256299,
        # Heights of aircraft sections.
        'height_nose_section_01': 0.647944796,
        'height_nose_section_02': 1.283759313,
        'height_nose_section_03': 1.894925056,
        'height_nose_section_04': 2.468092813,
        'height_nose_section_05': 2.988514422,
        'height_nose_section_06': 3.439232935,
        'height_nose_section_07': 3.799813925,
        'height_nose_section_08': 4.044157799,
        'height_nose_section_09': 4.136281495,
        'height_constant_section_01': 4.136281495,
        'height_tail_section_01': 4.074891643,
        'height_tail_section_02': 3.889781167,
        'height_tail_section_03': 3.578053688,
        'height_tail_section_04': 3.143415874,
        'height_tail_section_05': 2.681089784,
        'height_tail_section_06': 1.754173041,
        'height_tail_section_07': 0.827256299
    }
    # END
    dict_design['energy_carrier'] = routing_dict['user_layer']

    # Design tank(s).
    runtime_output.print('----------------------------------------------------------')
    runtime_output.print('Tank design results for design mission.')
    runtime_output.print('----------------------------------------------------------')
    dict_tank_design = tank_design_tu_berlin(paths_and_names, routing_dict, dict_design, runtime_output)

    # TODO: Geometry in 'dict_tank_design' necessary?
    # Set iteration variables.
    tank_idx = 0
    energy_available = 0
    mass_tank_overall = 0
    tank_location_list = []
    tank_position_list = []
    length_tank_overall = 0

    # Extract all tank entities from 'dict_tank_design'.
    tank_entity_list = [key for key in dict_tank_design if key.startswith('tank_')]
    # Check, if there is a tank in the tailcone of the aircraft.
    tailcone_tank = next((key for key in tank_entity_list if dict_tank_design[key]['tank_position'] == 'tailcone'),
                         None)
    # Sort list such that the key of the tank with the tailcone position is at the first list position. This ensures
    # that this tank is calculated first.
    if tailcone_tank:
        tank_entity_list.remove(tailcone_tank)
        tank_entity_list.insert(0, tailcone_tank)
    number_of_tanks = len(tank_entity_list)

    mission_energy_amount = dict_design['mission_energy_amount']['mission_energy_amount_ID0']

    # Iterate through tank entities and call functions from liquid hydrogen folder.
    for tank_entity in tank_entity_list:
        print('Calling liquid hydrogen functions for ...')
        tank_location_list.append(dict_tank_design[tank_entity]['tank_location'])
        tank_position_list.append(dict_tank_design[tank_entity]['tank_position'])

        # Call functions according to tank shape.
        match dict_tank_design[tank_entity]['tank_shape']:
            case 'conical':
                print('... a conical tank.')
                dict_tank_design = calculate_conical_tank(tank_entity, dict_design, dict_tank_design, runtime_output)
            case 'cylindrical':
                print('... a cylindrical tank.')
                dict_tank_design = calculate_cylindrical_tank(tank_entity, dict_design, dict_tank_design, runtime_output)

        energy_available += dict_tank_design[tank_entity]['energy_available']
        energy_remaining = mission_energy_amount - energy_available

        # Set missing entries in 'dict_tank_design'.
        if dict_tank_design[tank_entity]['tank_position'] == 'tailcone':
            dict_tank_design[tank_entity]['energy_required'] = dict_tank_design[tank_entity]['energy_available']
            if number_of_tanks == 2:
                # Da der Tank im tailcone immer zuerst berechnet wird und es nur einen gibt, muss bei einer Gesamtzahl
                # von zwei Tanks der gesamte zweite Tank das restliche Volumen beinhalten.
                dict_tank_design[tank_entity_list[-1]]['energy_required'] = energy_remaining
            else:
                # ...
                for tmp_tank in tank_entity_list[1:]:
                    if (dict_tank_design[tmp_tank]['tank_location'] == 'fuselage'
                        and dict_tank_design[tmp_tank]['tank_position'] == 'rear'):
                        # Tank at rear fuselage.
                        # Assumption: energy_share von hinterem Tank beinhaltet energy des tailcone Tanks.
                        # energy_required = (mission_energy*energy_share) - energy_tailcone_tank
                        dict_tank_design[tmp_tank]['energy_required'] = (
                            mission_energy_amount*dict_tank_design[tmp_tank]['energy_share']
                            - dict_tank_design[tank_entity]['energy_available'])
                    elif (dict_tank_design[tmp_tank]['tank_location'] == 'fuselage'
                        and dict_tank_design[tmp_tank]['tank_position'] == 'front'):
                        # Tank at front fuselage.
                        dict_tank_design[tmp_tank]['energy_required'] = (
                            mission_energy_amount*dict_tank_design[tmp_tank]['energy_share'])

        length_tank_overall += dict_tank_design[tank_entity]['geometry']['length_tank_overall']
        mass_tank_overall += dict_tank_design[tank_entity]['masses']['mass_tank_overall']

        if energy_available > mission_energy_amount:
            # TODO: Check for unnecessary tank entities.
            print('Necessary volume available.')
            break
        if energy_available < mission_energy_amount and (tank_idx+1) == number_of_tanks:
            # Sollte eigentlich nicht vorkommen, weil Tanks so dimensioniert werden, dass alles reinpasst.
            print('Error: Necessary volume not available!')

        tank_idx += 1

    # Add extra space between tanks.
    # Check, if all tanks in fuselage.
    if len(tank_location_list) > 0 and all(element == 'fuselage' for element in tank_location_list):
        if len(tank_position_list) > 0 and all(element in {'tailcone', 'rear'} for element in tank_position_list):
            # Add extra space between tanks in rear and tailcone of fuselage.
            extra_space = 0.2
            length_tank_overall = length_tank_overall + extra_space

    liquid_hydrogen_output_dict = dict_tank_design

    return liquid_hydrogen_output_dict
