"""
  MassProperties IO
"""


class MassPropertiesIO:
    def __init__(self, path_to_element=None):
        self.path_to_element = path_to_element

        self.mass = None
        self.name = None
        self.center_of_gravity = {
            "x": None,
            "y": None,
            "z": None
        }

        self.inertia = {
            "j_xx": None,
            "j_yy": None,
            "j_zz": None,
            "j_xy": None,
            "j_xz": None,
            "j_yx": None,
            "j_yz": None,
            "j_zx": None,
            "j_zy": None
        }

    def initialize_zero(self):
        """Initialize to empty

        Returns:
            MassPropertiesIO Object: self
        """
        self.path_to_element = None
        self.initialize_zero_mass()
        self.initialize_zero_center_of_gravity()
        self.initialize_zero_inertia()
        return self

    def initialize_zero_mass(self):
        """Initialize mass to zero

        Returns:
            MassPropertiesIO object: self
        """
        self.mass = 0.0
        return self

    def initialize_zero_center_of_gravity(self):
        """Initialize center_of_gravity to zero

        Returns:
            MassPropertiesIO object: self
        """
        for ax in self.center_of_gravity:
            self.center_of_gravity[ax] = 0.0
        return self

    def initialize_zero_inertia(self):
        """Initialize inertia to zero

        Returns:
            MassPropertiesIO object: self
        """
        for component in self.inertia:
            self.inertia[component] = 0.0
        return self

    def read(self, xml_access):
        """Read from xml node

        Args:
            xml_access (elementtree node): access point to elementtree (normally root)

        Returns:
            MassPropertiesIO Object: self
        """
        if self.path_to_element is not None:
            self.mass = float(xml_access.find(
                f"{self.path_to_element}/mass/value").text)
            for key in self.inertia:
                self.inertia[key] = float(xml_access.find(
                    f"{self.path_to_element}/inertia/{key}/value").text)

            for key in self.center_of_gravity:
                self.center_of_gravity[key] = float(xml_access.find(
                    f"{self.path_to_element}/center_of_gravity/{key}/value").text)
        else:
            raise ValueError
        return self

    def print(self, name=None):

        import logging
        runtime_output = logging.getLogger('module_logger')

        if name is not None:
            runtime_output.print(f"== {name} == ")
        runtime_output.print(f"mass ... {self.mass}")
        for ax, value in self.center_of_gravity.items():
            runtime_output.print(f"{ax} ... {value}")

        for component, value in self.inertia.items():
            runtime_output.print(f"{component} ... {value}")

        return self

    def user_method_data_generation(self):
        element_name = self.path_to_element.split("/")[-1]
        to_write = {
            f"{element_name}_mass": [f"{self.path_to_element}/mass_properties/mass", self.mass],
            f"{element_name}_cog_x": [f"{self.path_to_element}/mass_properties/center_of_gravity/x",
                                      self.center_of_gravity["x"]],
            f"{element_name}_cog_y": [f"{self.path_to_element}/mass_properties/center_of_gravity/y",
                                      self.center_of_gravity["y"]],
            f"{element_name}_cog_z": [f"{self.path_to_element}/mass_properties/center_of_gravity/z",
                                      self.center_of_gravity["z"]],
            f"{element_name}_j_xx": [f"{self.path_to_element}/mass_properties/inertia/j_xx", self.inertia["j_xx"]],
            f"{element_name}_j_yy": [f"{self.path_to_element}/mass_properties/inertia/j_yy", self.inertia["j_yy"]],
            f"{element_name}_j_zz": [f"{self.path_to_element}/mass_properties/inertia/j_zz", self.inertia["j_zz"]],
            f"{element_name}_j_xy": [f"{self.path_to_element}/mass_properties/inertia/j_xy", self.inertia["j_xy"]],
            f"{element_name}_j_xz": [f"{self.path_to_element}/mass_properties/inertia/j_xz", self.inertia["j_xz"]],
            f"{element_name}_j_yx": [f"{self.path_to_element}/mass_properties/inertia/j_yx", self.inertia["j_yx"]],
            f"{element_name}_j_yz": [f"{self.path_to_element}/mass_properties/inertia/j_yz", self.inertia["j_yz"]],
            f"{element_name}_j_zx": [f"{self.path_to_element}/mass_properties/inertia/j_zx", self.inertia["j_zx"]],
            f"{element_name}_j_zy": [f"{self.path_to_element}/mass_properties/inertia/j_zy", self.inertia["j_zy"]]
        }
        return to_write

    def key_parameters_paths(self, mass_type_w_underscore: str = "mass"):

        basic_path = f"./analysis/masses_cg_inertia/{mass_type_w_underscore}/mass_properties"
        mass_item_path = f"{basic_path}/mass"
        inertia_path_j_xx = f"{basic_path}/inertia/j_xx"
        inertia_path_j_yy = f"{basic_path}/inertia/j_yy"
        inertia_path_j_zz = f"{basic_path}/inertia/j_zz"
        inertia_path_j_xy = f"{basic_path}/inertia/j_xy"
        inertia_path_j_xz = f"{basic_path}/inertia/j_xz"
        inertia_path_j_yx = f"{basic_path}/inertia/j_yx"
        inertia_path_j_yz = f"{basic_path}/inertia/j_yz"
        inertia_path_j_zx = f"{basic_path}/inertia/j_zx"
        inertia_path_j_zy = f"{basic_path}/inertia/j_zy"
        cog_path_x = f"{basic_path}/center_of_gravity/x"
        cog_path_y = f"{basic_path}/center_of_gravity/y"
        cog_path_z = f"{basic_path}/center_of_gravity/z"

        return [mass_item_path,
                inertia_path_j_xx, inertia_path_j_yy, inertia_path_j_zz,
                inertia_path_j_xy, inertia_path_j_xz,
                inertia_path_j_yx, inertia_path_j_yz,
                inertia_path_j_zx, inertia_path_j_zy,
                cog_path_x, cog_path_y, cog_path_z]

    def key_parameters_dicts(self, mass_type_w_underscore: str = "mass", short: str = "M"):
        mass_type_wo_underscore = mass_type_w_underscore.replace("_", " ")

        parameters_dict = {
            f"{mass_type_w_underscore}": {
                "attributes": {"description": f"{short}"},
                "mass_properties": {
                    "attributes": {"description": f"{mass_type_wo_underscore} properties"},
                    'mass': {
                        'attributes': {'description': 'Mass'},
                        'value': '0',
                        'unit': 'kg',
                                'lower_boundary': '0',
                                'upper_boundary': 'inf'},
                    'inertia': {
                        'attributes': {
                            'description': 'Inertia with regard to the total center of gravity.'},
                        'j_xx': {
                            'attributes': {
                                'description': 'Inertia in x.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_yy': {
                            'attributes': {
                                'description': 'Inertia in y.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_zz': {
                            'attributes': {
                                'description': 'Inertia in z.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_xy': {
                            'attributes': {
                                'description': 'Inertia in xy.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_xz': {
                            'attributes': {
                                'description': 'Inertia in xz.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_yx': {
                            'attributes': {
                                'description': 'Inertia in yx.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_yz': {
                            'attributes': {
                                'description': 'Inertia in yz.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_zx': {
                            'attributes': {
                                'description': 'Inertia in zx.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                        'j_zy': {
                            'attributes': {
                                'description': 'Inertia in zy.'},
                            'value': '0',
                            'unit': 'kgm^2',
                            'lower_boundary': '-inf',
                            'upper_boundary': 'inf'},
                    },
                    'center_of_gravity': {
                        'attributes': {
                            'description': 'Center of gravity w.r.t global coordinate system.'},
                        'x': {
                            'attributes': {
                                'description':
                                'Center of gravity in x-direction with regard to the global reference point.'},
                            'value': '0',
                            'unit': 'm',
                                    'lower_boundary': '-100',
                                    'upper_boundary': '100'},
                        'y': {
                            'attributes': {
                                'description':
                                'Center of gravity in y-direction with regard to the global reference point.'},
                            'value': '0',
                            'unit': 'm',
                                    'lower_boundary': '-50',
                                    'upper_boundary': '50'},
                        'z': {
                            'attributes': {
                                'description':
                                'Center of gravity in z-direction with regard to the global reference point.'},
                            'value': '0',
                            'unit': 'm',
                                    'lower_boundary': '-10',
                                    'upper_boundary': '10'}
                    }
                }
            }
        }

        return parameters_dict
